#ifndef _ADATA_COMMON_H
#define _ADATA_COMMON_H

#include <sys/time.h>

#include <admxrc2.h>

/*
** Define some types for manipulating hardware
*/

typedef int            BOOLEAN;

#if !defined(FALSE)
#   define FALSE (0)
#endif

#if !defined(TRUE)
#   define TRUE (1)
#endif

/*
** ---------------------------------------------------------------------------
** Functions and types for parsing command line arguments
** ---------------------------------------------------------------------------
*/

typedef enum _OptionType
{
  Option_float   = 0x0,
  Option_double  = 0x1,
  Option_int     = 0x2,
  Option_uint    = 0x3,
  Option_hex     = 0x4,
  Option_string  = 0x5,
  Option_boolean = 0x6,
  Option_path    = 0x7
} OptionType;

typedef union _Value
{
  float            floatVal;
  double           doubleVal;
  long             intVal;
  unsigned long    uintVal;
  unsigned long    hexVal;
  char*            stringVal;
  BOOLEAN          booleanVal;
  char*            pathVal;
} Value;

typedef struct _Option
{
  /* Must be initialised before call */
  const char*      key;
  int              type;
  const char*      meaning;
  union _Value     def;
  /* Filled in on return */
  BOOLEAN          specified;
  union _Value     value;
  int              nParam;
  union _Value*    param;
} Option;

typedef struct _Arguments
{
  /* Initialised before call */
  int              nOption;
  OptionType*      paramType;
  char**           paramInfo;
  char**           paramMeaning;
  int              nParamType;
  int              minNParam;
  /* Partially initialised before call */
  Option*          option;
  /* Filled in on return */
  int              nParam;
  Value*           param;

  char*            appName;
  char*            sdkPath;
  char*            bitPath;

  char*            bitFile;
} Arguments;

int
parse_command_line(
  char*                appName,
  int                  argc,
  char*                argv[],
  Arguments*           args);

const char*
get_fpga_filename(
  ADMXRC2_CARD_INFO*   cardInfo,
  const char*          design,
  Arguments*           args);

const char*
get_fpga_filename_dir(
  ADMXRC2_CARD_INFO*   cardInfo,
  const char*          dir,
  const char*          design,
  Arguments*           args);

const char*
get_fpga_filename_ext(
  ADMXRC2_CARD_INFO*   cardInfo,
  unsigned int         targetIndex,
  const char*          dir,
  const char*          design,
  Arguments*           args);

void
dump_options(
  Arguments*           args);

void
dump_command_line(
  Arguments*           args);

/*
** ---------------------------------------------------------------------------
** Functions and types for handing values of type "timeval"
** ---------------------------------------------------------------------------
*/

struct timeval
get_time(void);

struct timeval
make_time(long sec, long usec);

int
time_after(
  const struct timeval* t1,
  const struct timeval* t2);

int
time_before(
  const struct timeval* t1,
  const struct timeval* t2);

int
time_equal(
  const struct timeval* t1,
  const struct timeval* t2);

struct timeval
time_add(
  const struct timeval* t1,
  const struct timeval* t2);

struct timeval
time_subtract(
  const struct timeval* t1,
  const struct timeval* t2);

double
time_to_double(
  const struct timeval* t1);

float
time_to_float(
  const struct timeval* t1);

struct timeval
double_to_time(
  double t1);

struct timeval
float_to_time(
  float t1);

int
sleep_ms(
  unsigned long ms);

#endif
