#ifndef _ADATA_COMMON_PLATFORM_H
#define _ADATA_COMMON_PLATFORM_H

/*
** platform.h - platform abstraction header file
*/

#include <endian.h>

/* Only use this if the system does not supply appropriate endian macros */
#ifndef cpu_to_le32

#define ___swab8(x) (x)

#define ___swab16(x) \
({ \
	uint16_t __x = (x); \
	((uint16_t)( \
		(((uint16_t)(__x) & (uint16_t)0x00ffU) << 8) | \
		(((uint16_t)(__x) & (uint16_t)0xff00U) >> 8) )); \
})

#define ___swab32(x) \
({ \
	uint32_t __x = (x); \
	((uint32_t)( \
		(((uint32_t)(__x) & (uint32_t)0x000000ffUL) << 24) | \
		(((uint32_t)(__x) & (uint32_t)0x0000ff00UL) <<  8) | \
		(((uint32_t)(__x) & (uint32_t)0x00ff0000UL) >>  8) | \
		(((uint32_t)(__x) & (uint32_t)0xff000000UL) >> 24) )); \
})

#define ___swab64(x) \
({ \
	uint64_t __x = (x); \
	((uint64_t)( \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000000000ffULL) << 56) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000000000ff00ULL) << 40) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000000000ff0000ULL) << 24) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000ff000000ULL) <<  8) | \
	        (uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000ff00000000ULL) >>  8) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000ff0000000000ULL) >> 24) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00ff000000000000ULL) >> 40) | \
		(uint64_t)(((uint64_t)(__x) & (uint64_t)0xff00000000000000ULL) >> 56) )); \
})

/* Swap endian conversion functions to use */
#if __BYTE_ORDER == __BIG_ENDIAN
#define os_cpu_to_le64(x) ___swab64((x))
#define os_le64_to_cpu(x) ___swab64((x))
#define os_cpu_to_le32(x) ___swab32((x))
#define os_le32_to_cpu(x) ___swab32((x))
#define os_cpu_to_le16(x) ___swab16((x))
#define os_le16_to_cpu(x) ___swab16((x))
#define os_cpu_to_le8(x) ___swab8((x))
#define os_le8_to_cpu(x) ___swab8((x))
#elif __BYTE_ORDER == __LITTLE_ENDIAN
#define os_cpu_to_le64(x) (x)
#define os_le64_to_cpu(x) (x)
#define os_cpu_to_le32(x) (x)
#define os_le32_to_cpu(x) (x)
#define os_cpu_to_le16(x) (x)
#define os_le16_to_cpu(x) (x)
#define os_cpu_to_le8(x) (x)
#define os_le8_to_cpu(x) (x)
#else
#  error Cannot determine endianness of this platform!
#endif

#else  /* end of defined(cpu_to_le32) */

/*
** System supplies appropriate endian conversion macros
*/

#define os_cpu_to_le64(x) cpu_to_le64(x)
#define os_le64_to_cpu(x) le64_to_cpu(x)
#define os_cpu_to_le32(x) cpu_to_le32(x)
#define os_le32_to_cpu(x) le32_to_cpu(x)
#define os_cpu_to_le16(x) cpu_to_le16(x)
#define os_le16_to_cpu(x) le16_to_cpu(x)
#define os_cpu_to_le8(x) (x)
#define os_le8_to_cpu(x) (x)

#endif /* end of !defined(cpu_to_le32) */

#endif

