/*
** time.c - Utility functions for handling variables of type "timeval"
**
**          Used by the example applications.
*/

#include <stdio.h>
#include <stdlib.h>

#include "common.h"

struct timeval
get_time(void)
{
  struct timeval t;
  int err;

  err = gettimeofday(&t, NULL);
  if (err) {
    printf("*** get_time: gettimeofday failed\n");
    exit(-1);
  }

  return t;
}

struct timeval
make_time(long seconds, long us)
{
  struct timeval t;

  t.tv_sec = seconds;
  t.tv_usec = us;

  return t;
}

double
time_to_double(
  const struct timeval* t1)
{
  return
    ((double)t1->tv_usec) / 1000000.0 +
    ((double)t1->tv_sec);
}

float
time_to_float(
  const struct timeval* t1)
{
  return
    ((float)t1->tv_usec) / 1000000.0f +
    ((float)t1->tv_sec);
}

struct timeval
double_to_time(
  double t1)
{
  struct timeval t;

  t.tv_sec = (long)t1;
  t.tv_usec = (long)((t1 - (double)t.tv_sec) * 1000000.0);

  return t;
}

struct timeval
float_to_time(
  float t1)
{
  struct timeval t;

  t.tv_sec = (long)t1;
  t.tv_usec = (long)((t1 - (float)t.tv_sec) * 1000000.0f);

  return t;
}

int
time_after(
  const struct timeval* t1,
  const struct timeval* t2)
{
  if (t1->tv_sec > t2->tv_sec) {
    return 1;
  } else {
    if (t1->tv_sec == t2->tv_sec) {
      return (t1->tv_usec > t2->tv_usec);
    } else {
      return 0;
    }
  }
}

int
time_equal(
  const struct timeval* t1,
  const struct timeval* t2)
{
  return (t1->tv_sec == t2->tv_sec && t1->tv_usec == t2->tv_usec);
}

int
time_before(
  const struct timeval* t1,
  const struct timeval* t2)
{
  if (t1->tv_sec < t2->tv_sec) {
    return 1;
  } else {
    if (t1->tv_sec == t2->tv_sec) {
      return (t1->tv_usec < t2->tv_usec);
    } else {
      return 0;
    }
  }
}

struct timeval
time_add(
  const struct timeval* t1,
  const struct timeval* t2)
{
  struct timeval t;

  t.tv_usec = t1->tv_usec + t2->tv_usec;
  t.tv_sec = t1->tv_sec + t2->tv_sec;
  if (t.tv_usec >= 1000000) {
    t.tv_usec -= 1000000;
    t.tv_sec++;
  }

  return t;
}

struct timeval
time_subtract(
  const struct timeval* t1,
  const struct timeval* t2)
{
  struct timeval t;

  t.tv_usec = t1->tv_usec - t2->tv_usec;
  t.tv_sec = t1->tv_sec - t2->tv_sec;
  if (t.tv_usec < 0) {
    t.tv_usec += 1000000;
    t.tv_sec--;
  }

  return t;
}

int
sleep_ms(
  unsigned long ms)
{
  struct timeval to;
  fd_set zero;

  FD_ZERO(&zero);
  to.tv_sec = ms / 1000;
  to.tv_usec = (ms % 1000) * 1000;
  return select(0, &zero, &zero, &zero, &to) == 0;
}
