/*
** info.c - utility to return information about an ADM-XRC series card.
**
** Example for:
**
**   o ADM-XRC
**   o ADM-XRC-P
**   o ADM-XRC-II-Lite
**   o ADM-XRC-II
**   o ADM-XPL
**   o ADM-XP
**   o ADP-WRC-II
**   o ADP-DRC-II
**   o ADP-XPI
**   o ADM-XRC-4LX
**   o ADM-XRC-4SX
**   o ADM-XRC-4FX
**   o ADPE-XRC-4FX
**   o ADM-XRC-5LX
**   o ADM-XRC-5T1
**   o ADM-XRC-5T2 / ADM-XRC-5T2-ADV
**   o ADM-XRC-5TZ
**   o ADM-XRC-5T-DA1
**
** (C) Copyright 2000-2009 Alpha Data
*/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>

#include <admxrc2.h>

#include "../common/common.h"
#include "../common/platform.h"

static char bnames[][32] = {
  "ADM-XRC",
  "ADM-XRC-P",
  "ADM-XRC-II-Lite",
  "ADM-XRC-II",
  "ADM-XP",
  "ADM-XPL",
  "ADP-WRC-II",
  "ADP-DRC-II",
  "ADP-XPI",
  "ADM-XRC-4LS",
  "ADM-XRC-4LX",
  "ADM-XRC-4SX",
  "ADPE-XRC-4FX",
  "ADM-XRC-4FX",
  "ADM-XRC-5LX",
  "ADM-XRC-5T1",
  "ADM-XRC-5T2",
  "ADCP-XRC-4LX",
  "ADM-AMC-5A2",
  "ADM-XRC-5TZ",
  "ADC-BBP",
  "ADM-XRC-5T2-ADV",
  "ADM-XRC-5T-DA1",
};

static char fnames[][32] = {
  "XC4085XL",
  "XC40150XV",
  "XC40200XV",
  "XC40250XV", 
  "Virtex V1000",
  "Virtex V400",
  "Virtex V600",
  "Virtex V800", 
  "Virtex V2000E",
  "Virtex V1000E",
  "Virtex V1600E",
  "Virtex V3200E",
  "Virtex V812E",
  "Virtex V405E",
  "Unknown14",
  "Unknown15",
  "Unknown16",
  "Unknown17",
  "Unknown18",
  "Unknown19",
  "Unknown20",
  "Unknown21",
  "Unknown22",
  "Unknown23",
  "Unknown24",
  "Unknown25",
  "Unknown26",
  "Unknown27",
  "Unknown28",
  "Unknown29",
  "Unknown30",
  "Unknown31",
  "Virtex-II 2V1000",
  "Virtex-II 2V1500",
  "Virtex-II 2V2000",
  "Virtex-II 2V3000",
  "Virtex-II 2V4000",
  "Virtex-II 2V6000",
  "Virtex-II 2V8000",
  "Virtex-II 2V10000",
  "Unknown40",
  "Unknown41",
  "Unknown42",
  "Unknown43",
  "Unknown44",
  "Unknown45",
  "Unknown46",
  "Unknown47",
  "Unknown48",
  "Unknown49",
  "Unknown50",
  "Unknown51",
  "Unknown52",
  "Unknown53",
  "Unknown54",
  "Unknown55",
  "Unknown56",
  "Unknown57",
  "Unknown58",
  "Unknown59",
  "Unknown60",
  "Unknown61",
  "Unknown62",
  "Unknown63",
  "Virtex-II Pro 2VP2",
  "Virtex-II Pro 2VP4",
  "Virtex-II Pro 2VP7",
  "Virtex-II Pro 2VP20",
  "Virtex-II Pro 2VP30",
  "Virtex-II Pro 2VP40",
  "Virtex-II Pro 2VP50",
  "Virtex-II Pro 2VP100",
  "Virtex-II Pro 2VP125",
  "Virtex-II Pro 2VP70",
  "Unknown74",
  "Unknown75",
  "Unknown76",
  "Unknown77",
  "Unknown78",
  "Unknown79",
  "Unknown80",
  "Unknown81",
  "Unknown82",
  "Unknown83",
  "Unknown84",
  "Unknown85",
  "Unknown86",
  "Unknown87",
  "Unknown88",
  "Unknown89",
  "Unknown90",
  "Unknown91",
  "Unknown92",
  "Unknown93",
  "Unknown94",
  "Unknown95",
  "Virtex-4 LX15",
  "Virtex-4 LX25",
  "Virtex-4 LX40",
  "Virtex-4 LX60",
  "Virtex-4 LX100",
  "Virtex-4 LX160",
  "Virtex-4 LX200",
  "Virtex-4 LX80",
  "Virtex-4 SX25",
  "Virtex-4 SX35",
  "Virtex-4 SX55",
  "Unknown107",
  "Unknown108",
  "Unknown109",
  "Unknown110",
  "Unknown111",
  "Virtex-4 FX12",
  "Virtex-4 FX20",
  "Virtex-4 FX40",
  "Virtex-4 FX60",
  "Virtex-4 FX100",
  "Virtex-4 FX140",
  "Unknown118",
  "Unknown119",
  "Unknown120",
  "Unknown121",
  "Unknown122",
  "Unknown123",
  "Unknown124",
  "Unknown125",
  "Unknown126",
  "Unknown127",
  "Virtex-5 V5LX30",
  "Virtex-5 V5LX50",
  "Virtex-5 V5LX85",
  "Virtex-5 V5LX110",
  "Virtex-5 V5LX220",
  "Virtex-5 V5LX330",
  "Unknown134",
  "Unknown135",
  "Virtex-5 V5LX30T",
  "Virtex-5 V5LX50T",
  "Virtex-5 V5LX85T",
  "Virtex-5 V5LX110T",
  "Virtex-5 V5LX330T",
  "Virtex-5 V5LX220T",
  "Virtex-5 V5LX155T",
  "Unknown143",
  "Virtex-5 V5SX35T",
  "Virtex-5 V5SX50T",
  "Virtex-5 V5SX95T",
  "Virtex-5 V5SX240T",
  "Unknown148",
  "Unknown149",
  "Unknown150",
  "Unknown151",
  "Virtex-5 V5FX100T",
  "Virtex-5 V5FX130T",
  "Virtex-5 V5FX200T",
  "Virtex-5 V5FX30T",
  "Virtex-5 V5FX70T"
};

static const char*
spacename(
  int                index,
  ADMXRC2_BOARD_TYPE boardtype)
{
  if (index == 0) {
    return "FPGA";
  } else {
    switch (boardtype) {
    case ADMXRC2_BOARD_ADMXRC:
    case ADMXRC2_BOARD_ADMXRC_P:
    case ADMXRC2_BOARD_ADMXRC2_LITE:
    case ADMXRC2_BOARD_ADMXRC2:
    case ADMXRC2_BOARD_ADMXPL:
    case ADMXRC2_BOARD_ADMXP:
    case ADMXRC2_BOARD_ADPDRC2:
    case ADMXRC2_BOARD_ADPWRC2:
    case ADMXRC2_BOARD_ADPXPI:
    case ADMXRC2_BOARD_ADMXRC4LS:
    case ADMXRC2_BOARD_ADMXRC4LX:
    case ADMXRC2_BOARD_ADMXRC4SX:
    case ADMXRC2_BOARD_ADPEXRC4FX:
    case ADMXRC2_BOARD_ADMXRC4FX:
    case ADMXRC2_BOARD_ADMXRC5LX:
    case ADMXRC2_BOARD_ADMXRC5T1:
    case ADMXRC2_BOARD_ADMXRC5T2:
    case ADMXRC2_BOARD_ADCPXRC4LX:
    case ADMXRC2_BOARD_ADMXRC5TZ:
    case ADMXRC2_BOARD_ADCBBP:
    case ADMXRC2_BOARD_ADMXRC5T2ADV:
    case ADMXRC2_BOARD_ADMXRC5TDA1:
      if (index == 1) {
	return "control";
      } else {
	return "unknown!";
      }
    default:
      return "unknown!";
    }
  }
}

int
main(int argc, char * argv[])
{
  int                  ret = 0;
  ADMXRC2_HANDLE       card = ADMXRC2_HANDLE_INVALID_VALUE;
  ADMXRC2_STATUS       status;
  ADMXRC2_VERSION_INFO versionInfo;
  ADMXRC2_CARD_INFO    cardInfo;
  ADMXRC2_FPGA_INFO    fpgaInfo;
  ADMXRC2_SPACE_INFO*  spaces;
  char                 fname[256];
  char                 bname[256];
  uint32_t             i;
  Option               options[2];
  Arguments            arguments;
	
  options[0].key = "card";
  options[0].type = Option_uint;
  options[0].def.uintVal = 0;
  options[0].meaning = "ID of card to use";
  options[1].key = "index";
  options[1].type = Option_uint;
  options[1].def.uintVal = 0;
  options[1].meaning = "index of card to use";
  arguments.nOption = 2;
  arguments.option = options;
  arguments.nParamType = 0;
  arguments.minNParam = 0;
	
  parse_command_line("info",
		     argc,
		     argv,
		     &arguments);

  if (options[0].specified && options[1].specified) {
    printf("Do not specify both /card and /index\n");
    ret = -1;
    goto done;
  }

  if (!options[1].specified) {
    ADMXRC2_CARDID cardID = options[0].value.uintVal;

    status = ADMXRC2_OpenCard(cardID, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("Failed to open card with ID %ld: %s\n",
	     (unsigned long) cardID, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  } else {
    unsigned int index = options[1].value.uintVal;

    status = ADMXRC2_OpenCardByIndex(index, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("Failed to open card with index %ld: %s\n",
	     (unsigned long) index, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  }

  status = ADMXRC2_GetCardInfo(card, &cardInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("Failed to get card info: %s\n",
	   ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  status = ADMXRC2_GetVersionInfo(card, &versionInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("Failed to get version info: %s\n",
	   ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  printf("API ver                      %ld.%ld\n",
	 (unsigned long) versionInfo.APIMajor,
	 (unsigned long) versionInfo.APIMinor);
  printf("Driver ver                   %ld.%ld\n",
	 (unsigned long) versionInfo.DriverMajor,
	 (unsigned long) versionInfo.DriverMinor);
  printf("\n");

  if (cardInfo.BoardType < 0 || cardInfo.BoardType >= ADMXRC2_BOARD_UNKNOWN) {
    sprintf(bname, "unknown (0x%lx)", (unsigned long) cardInfo.BoardType);
  } else {
    strncpy(bname, bnames[cardInfo.BoardType], 255);
  }
  printf("Board Type                   %s\n", bname);
  printf("Card ID                      %ld (0x%4.4lX)\n",
	 (unsigned long) cardInfo.CardID,
	 (unsigned long) cardInfo.CardID);
  printf("Serial Number                %ld (0x%8.8lX)\n",
	 (unsigned long) cardInfo.SerialNum,
	 (unsigned long) cardInfo.SerialNum);
  printf("Board/Logic Rev              %d.%d/%d.%d\n",
	 cardInfo.BoardRevision >> 4, cardInfo.BoardRevision & 0xF,
	 cardInfo.LogicRevision >> 4, cardInfo.LogicRevision & 0xF);
  for (i = 0;; i++) {
    status = ADMXRC2_GetFPGAInfo(card, i, &fpgaInfo);
    if (status == ADMXRC2_SUCCESS) {
      if (fpgaInfo.Type < 0 || fpgaInfo.Type >= ADMXRC2_FPGA_UNKNOWN) {
	sprintf(fname, "unknown (0x%lx)", (unsigned long)fpgaInfo.Type);
      } else {
	strncpy(fname, fnames[fpgaInfo.Type], 255);
      }
      printf("FPGA %-ld                       %s [%s]\n", (unsigned long)i, fname, fpgaInfo.Package);
      printf("\n");
    } else {
      if (0 == i) {
	printf("FPGA not present!");
      }
      break;
    }
  }

  printf("Number of clock generators   %ld\n",
	 (unsigned long) cardInfo.NumClock);
  printf("Number of DMA channels       %ld\n",
	 (unsigned long) cardInfo.NumDMAChan);
  printf("Number of spaces             %ld\n",
	 (unsigned long) cardInfo.NumSpace);
  printf("\n");

  spaces = (ADMXRC2_SPACE_INFO*) malloc(cardInfo.NumSpace * sizeof(ADMXRC2_SPACE_INFO));
  for (i = 0; i < cardInfo.NumSpace; i++) {
    unsigned long lsize, vsize;

    status = ADMXRC2_GetSpaceInfo(card, i, &spaces[i]);
    if (status != ADMXRC2_SUCCESS) {
      printf("Space %d: unable to get information\n", i);
    } else {
      printf("Space %d (%s):\n", i, spacename(i, cardInfo.BoardType));
      printf("Physical base                0x%8.8lX\n",
	     (unsigned long) spaces[i].PhysicalBase);
      lsize = spaces[i].LocalSize;
      if (lsize) {
	printf("Local range                  0x%8.8lX - 0x%8.8lX\n",
	       (unsigned long) spaces[i].LocalBase,
	       (unsigned long) (spaces[i].LocalBase + spaces[i].LocalSize - 1));
      }
      vsize = spaces[i].VirtualSize;
      if (vsize) {
	printf("Virtual range                0x%8.8lX - 0x%8.8lX\n",
	       (unsigned long) spaces[i].VirtualBase,
	       ((unsigned long) spaces[i].VirtualBase) + vsize - 1);
      }
    }
  }
  free(spaces);
  printf("\n");

  printf("Number of RAM banks          %ld\n",
	 (unsigned long) cardInfo.NumRAMBank);
  printf("Bank presence bitmap         0x%8.8lX\n",
	 (unsigned long) cardInfo.RAMBanksFitted);
  for (i = 0; i < cardInfo.NumRAMBank; i++) {
    ADMXRC2_BANK_INFO bank;

    printf("RAM Bank %02ld                  ", (unsigned long) i);
    status = ADMXRC2_GetBankInfo(card, i, &bank);
    if (status != ADMXRC2_SUCCESS) {
      printf("Cannot determine\n");
    } else {
      if (!bank.Fitted) {
	printf("Not fitted\n");
      } else {
	uint32_t size, byteWidth;
	char     type[256];
	int      ntype = 0;
				
	strcpy(type, "");
	if (bank.Type & ADMXRC2_RAM_ZBTFT) {
	  if (ntype) {
	    strcat(type, "+");
	  }
	  strcat(type, "ZBTFT");
	  ntype++;
	}
	if (bank.Type & ADMXRC2_RAM_ZBTP) {
	  if (ntype) {
	    strcat(type, "+");
	  }
	  strcat(type, "ZBTP");
	  ntype++;
	}
	if (bank.Type & ADMXRC2_RAM_SDRAM_DDR) {
	  if (ntype) {
	    strcat(type, "+");
	  }
	  strcat(type, "DDR SDRAM");
	  ntype++;
	}
	if (bank.Type & ADMXRC2_RAM_SRAM_DDR2) {
	  if (ntype) {
	    strcat(type, "+");
	  }
	  strcat(type, "DDR-II SRAM");
	  ntype++;
	}
	if (bank.Type & ADMXRC2_RAM_SDRAM_SDR) {
	  if (ntype) {
	    strcat(type, "+");
	  }
	  strcat(type, "SDR SDRAM");
	  ntype++;
	}
	if (bank.Type & ADMXRC2_RAM_SDRAM_DDR2) {
	  if (ntype) {
	    strcat(type, "+");
	  }
	  strcat(type, "DDR-II SDRAM");
	  ntype++;
	}
	if ((bank.Width % 9) == 0) {
	  /* Take care of x9 memories (with parity bits) */
	  byteWidth = bank.Width / 9;
	} else {
	  byteWidth = bank.Width / 8;
	}
	size = byteWidth * (bank.Size / 1024);
	printf("%s %ldkword x %ldbits (%ldkB)\n",
	       type,
	       (unsigned long) (bank.Size / 1024),
	       (unsigned long) bank.Width,
	       (unsigned long) size);
      }
    }
  }
  printf("\n");

 done:
  if (card != ADMXRC2_HANDLE_INVALID_VALUE) {
    ADMXRC2_CloseCard(card);
  }

  return ret;
}
