#
# Makefile for building ADM-XRC sample applications for
# Virtex/Virtex-E/Virtex-EM family.
#
# When MAKE is called with this Makefile, the following macros
# must be defined via the MAKE command line:
#
#       Variable                    Example value
#
#       DESIGN                      simple
#       DEVICE                      1000e
#       FAMILY                      ve
#       SPEED                       6
#       PKG                         bg560
#       TARGET                      xrc
#       UCF                         simple
#

OUTPUTDIR=./output
BUILDDIR=./build
NETLISTDIR=./edif

BITDIR=$(ADMXRC_SDK4)/bit
INSTALLDIR=$(BITDIR)/$(DESIGN)

PARTTYPE=$(DEVICE)-$(SPEED)-$(PKG)

NETLIST=$(NETLISTDIR)/$(DESIGN)-$(TARGET)-$(FAMILY).ngc
CONSTRAINTS=$(UCF).ucf
BUILDNAME=$(BUILDDIR)/$(DESIGN)-$(TARGET)-$(DEVICE)
OUTPUTNAME=$(OUTPUTDIR)/$(DESIGN)-$(TARGET)-$(DEVICE)

all: $(OUTPUTNAME).ncd $(OUTPUTNAME).twr $(OUTPUTNAME).bit

clean:
	@rm -f $(OUTPUTNAME)*
	@rm -f $(BUILDNAME)*
	@rm -f $(BUILDDIR)/$(DESIGN)-$(TARGET)-$(FAMILY).ngo del $(BUILDDIR)/$(DESIGN)-$(TARGET)-$(FAMILY).ngo
	@rm -f $(DESIGN)-$(TARGET)-$(DEVICE)_ngdbuild.nav del $(DESIGN)-$(TARGET)-$(DEVICE)_ngdbuild.nav
	@echo Cleaned $(DESIGN)-$(TARGET)-$(DEVICE)

install: $(OUTPUTNAME).bit
	@cp -f $(OUTPUTNAME).bit $(INSTALLDIR)/$(DESIGN)-$(TARGET)-$(DEVICE).bit
	@echo Installed $(DESIGN)-$(TARGET)-$(DEVICE)

$(OUTPUTNAME).twr: $(OUTPUTNAME).ncd $(BUILDNAME).pcf
	@mkdir -p $(OUTPUTDIR)
	trce -e 3 -o $(OUTPUTNAME).twr $(OUTPUTNAME).ncd $(BUILDNAME).pcf

$(OUTPUTNAME).bit: $(OUTPUTNAME).ncd
	@mkdir -p $(OUTPUTDIR)
	bitgen -w -g drivedone:yes -g compress $(OUTPUTNAME).ncd $(OUTPUTNAME).bit

$(OUTPUTNAME).ncd: $(BUILDNAME)_map.ncd
	@mkdir -p $(OUTPUTDIR)
	par -ol high -w $(BUILDNAME)_map.ncd $(OUTPUTNAME).ncd $(BUILDNAME).pcf

$(BUILDNAME)_map.ncd: $(BUILDNAME).ngd
	@mkdir -p $(BUILDDIR)
	map -ol high -pr b -k 6 -o $(BUILDNAME)_map.ncd $(BUILDNAME).ngd $(BUILDNAME).pcf

$(BUILDNAME).ngd: $(NETLIST) $(CONSTRAINTS)
	@mkdir -p $(BUILDDIR)
	ngdbuild -nt on -sd ../chipscope/v -dd $(BUILDDIR) -uc $(CONSTRAINTS) -p $(PARTTYPE) $(NETLIST) $(BUILDNAME).ngd
	@rm -f $(DESIGN)-$(TARGET)-$(DEVICE)_ngdbuild.nav
