//
// clocks_dcm.v - clock multiplication module using DCMs for
//                Virtex-II
//
// (c) Alpha Data Parallel Systems Ltd. 2001
//
// The local bus clock is input and multiplied by 2 using a
// Virtex-II DCM.
//

module clocks(
   rst,
   lclk,
   mclk,
   clk,
   multclk,
   locked
);

input       rst;
wire        rst;
input       lclk;
wire        lclk;
input       mclk;
wire        mclk;
output      clk;
wire        clk;
output      multclk;
wire        multclk;
output      locked;
wire        locked;

wire        lclk_ibufg;
wire        lclk_bufg;
wire        clk0;
wire        multclk0;

reg         dllrst;
reg   [2:0] rstcnt;
    
wire        mclk_ibufg;
wire        mclk_bufg;

//
// Input MCLK (used for DCM reset, necessary for Virtex-IIES devices)
//

IBUFG    
ibufg_mclk(
   .I        (mclk),
   .O        (mclk_ibufg));

BUFG    
bufg_mclk(
   .I        (mclk_ibufg),
   .O        (mclk_bufg));

//
// Generate reset signal to DLLs/DCMs
//
always @ (posedge rst or posedge mclk_bufg)
begin
   if (rst) begin
      dllrst <= 1'b1;
      rstcnt <= 0;
   end else begin
      if (dllrst) begin
         rstcnt <= rstcnt + 1;
      end
      if (rstcnt == 3'h7) begin
         dllrst <= 1'b0;
      end
   end
end
    
//
// Input LCLK
//

IBUFG    
ibufg0(
   .I        (lclk),
   .O        (lclk_ibufg));

//
// Generate clocks locked to LCLK and LCLK x 2
//

DCM
dll0(
   .CLKIN    (lclk_ibufg),
   .CLKFB    (clk),
   .DSSEN    (1'b0),
   .PSINCDEC (1'b0),
   .PSEN     (1'b0),
   .PSCLK    (1'b0),
   .RST      (dllrst),
   .CLK0     (clk0),
   .CLK2X    (multclk0),
   .LOCKED   (locked));

/* synopsys attribute
DLL_FREQUENCY_MODE    "LOW"
DUTY_CYCLE_CORRECTION "TRUE"
STARTUP_WAIT          "FALSE"
*/

BUFG    
bufg0(
   .I        (clk0),
   .O        (clk));

BUFG
bufg1(
   .I        (multclk0),
   .O        (multclk));

endmodule
