/*
** frontio.vhd - FPGA design to output a walking 1 on I/O pins
**
** (C) Copyright Alpha Data Parallel Systems Ltd. 1999-2004
**
** Example FPGA design for:
**
**   o ADM-XRC
**   o ADM-XRCII-Lite
**   o ADM-XRCII
**
** This example outputs a walking bit on the 34 user I/O pins at
** the front panel connector.
*/

module frontio(
   lclk,
   lreset_l,
   frontio);

input           lclk;
input           lreset_l;
output [33:0]   frontio;

wire            rst = ~lreset_l;
reg    [33:0]   frontio;

//
// Shift pattern up from 0->33
//
// Use reset to initialise frontio<> to have bit 0 set.
//
always @ (posedge rst or posedge lclk)
begin
    if (rst) begin
        frontio <= 34'b1;
    end else begin
        frontio[33:0] <= { frontio[32:0], frontio[33] };
    end     
end

endmodule
