/*
** reario-xrcp.v - FPGA design to output a walking 1 on I/O pins
**
** (C) Copyright Alpha Data Parallel Systems Ltd. 1999-2004
**
** Example FPGA design for:
**
**   o ADM-XRC-P
**
** This example outputs a walking bit on the 53 user I/O pins at
** the Pn4 connector.
*/

module reario(
   lclk,
   lreset_l,
   reario);

input               lclk;
input               lreset_l;
inout  [52:0]       reario;

wire          	    rst = ~lreset_l;
wire   [52:0]       reario;

reg    [51:0]       shiftreg;
wire		    userclkin;

assign reario[50:0] = shiftreg[50:0]; // reario[50:0] are general purpose I/O
assign userclkin    = reario[51];     // reario[51] functions as user clock input
assign reario[51]   = 1'bZ;           // reario[51] isn't driven
assign reario[52]   = shiftreg[51];   // reario[52] is general purpose I/O

//
// Shift pattern up from 0 -> 51
//
always @ (posedge rst or posedge lclk)
begin
   if (rst) begin
      shiftreg <= 1;
   end else begin
      shiftreg[51:0] <= { shiftreg[50:0], shiftreg[51] } ;
   end     
end

endmodule

