//
// test_zbt.v - Testbench for entity 'zbt'
//
// (c) Alpha Data Parallel Systems Ltd. 1999-2001
//
// NOT synthesisable!
//

`timescale 1ns / 100ps

module test_zbt();

parameter max_bank = 8;

//
// These parameters don't correspond to any real card
//
parameter num_bank    = 4;
parameter num_clock   = 2;
parameter ctl_width   = 9;
parameter addr_width  = 20;
parameter data_width  = 16;
parameter bank_group  = 2;

parameter period      = 30;

parameter pctl_width  = ctl_width  * bank_group;
parameter paddr_width = addr_width * bank_group;
parameter pdata_width = data_width * bank_group;

parameter faddr_width = paddr_width * max_bank;
parameter fctl_width  = pctl_width  * max_bank;
parameter fdata_width = pdata_width * max_bank;

parameter ctl_bit_w   = (data_width / 8) + 0;
parameter ctl_bit_ce  = (data_width / 8) + 1;

reg                        lclk, mclk;
wire [num_clock - 1 : 0]   ramclk;
reg                        lreset_l;
reg                        lwrite;
reg                        lads_l;
reg                        lblast_l;
wire                       lbterm_l;
wire [31:0]                ld;
reg  [23:2]                la;
wire                       lready_l;
reg  [3:0]                 lbe_l;
wire                       global_lbo_l;
wire                       flothru_l;
reg                        fholda;

reg                        ld_oe;
reg  [31:0]                ld_o;

reg  [faddr_width - 1 : 0] del_ra;
reg  [fctl_width  - 1 : 0] del_rc;
wire [faddr_width - 1 : 0] ra;
wire [fctl_width  - 1 : 0] rc;
wire [fdata_width - 1 : 0] rd;
wire [fdata_width - 1 : 0] rd_o;
reg  [fdata_width - 1 : 0] rd_o0, rd_o1;
wire [bank_group * num_bank - 1 : 0] rd_oe;
reg  [bank_group * num_bank - 1 : 0] rd_oe0, rd_oe1;

reg  [31:0]                bank_data;
integer                    size_reg_val;
integer                    pipeline_reg_val;

function [data_width-1:0] get_data;
input [31:0]           bank;
input [31:0]           subbank;
input [addr_width-1:0] addr;
begin
    bank_data = bank * (1 << size_reg_val);
    bank_data = bank_data + (addr & ((1 << (size_reg_val + 17)) - 1));
    bank_data = bank_data >> (subbank * data_width);
    get_data  = bank_data[data_width - 1 : 0];
end
endfunction

always
begin
   #(period / 2) lclk = ~lclk;
end

always
begin
   #17 mclk = ~mclk;
end

//
// Drive data bus when we are writing to FPGA
//
assign ld = ld_oe ? ld_o : 32'hZ;

// Drive the SSRAM data busses on reads
assign rd_o  = pipeline_reg_val ? rd_o1 : rd_o0;
assign rd_oe = pipeline_reg_val ? rd_oe1 : rd_oe0;
assign rd[15:0]    = rd_oe[0] ? rd_o[15:0]    : 16'hZZZZ;
assign rd[31:16]   = rd_oe[1] ? rd_o[31:16]   : 16'hZZZZ;
assign rd[47:32]   = rd_oe[2] ? rd_o[47:32]   : 16'hZZZZ;
assign rd[63:48]   = rd_oe[3] ? rd_o[63:48]   : 16'hZZZZ;
assign rd[79:64]   = rd_oe[4] ? rd_o[79:64]   : 16'hZZZZ;
assign rd[95:80]   = rd_oe[5] ? rd_o[95:80]   : 16'hZZZZ;
assign rd[111:96]  = rd_oe[6] ? rd_o[111:96]  : 16'hZZZZ;
assign rd[127:112] = rd_oe[7] ? rd_o[127:112] : 16'hZZZZ;

always @ (ra or rc)
begin
    if (pipeline_reg_val) begin
        #(period*2) del_rc <= rc;
        #(period*2) del_ra <= ra;
    end else begin
        #(period*1) del_rc <= rc;
        #(period*1) del_ra <= ra;
    end
end

//
// Very simple behavioural model of the ZBT SSRAMs.
//
// Makes some assumptions about what is driven onto certain
// control pins of the SSRAMs.
//
always @ (posedge lreset_l or posedge lclk)
begin
    if (~lreset_l) begin
        rd_oe0 <= 0;
        rd_oe1 <= 0;
    end else begin

        rd_o1 <= rd_o0;
        rd_oe1 <= rd_oe0;

        /* Bank 0, subbank 0 */
        if (rc[5:4] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[15:0] <= get_data(0, 0, ra[19:0]);
                rd_oe0[0]   <= 1'b1;
            end else begin
                rd_o0[15:0] <= get_data(0, 0, ra[19:0]);
                rd_oe0[0]   <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[0] <= 1'b0;
            end else begin
                rd_oe0[0] <= 1'b0;
            end
        end

        /* Bank 0, subbank 1 */
        if (rc[14:13] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[31:16] <= get_data(0, 1, ra[39:20]);
                rd_oe0[1]    <= 1'b1;
            end else begin
                rd_o0[31:16] <= get_data(0, 1, ra[39:20]);
                rd_oe0[1]    <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[1] <= 1'b0;
            end else begin
                rd_oe0[1] <= 1'b0;
            end
        end

        /* Bank 1, subbank 0 */
        if (rc[23:22] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[47:32] <= get_data(1, 0, ra[59:40]);
                rd_oe0[2]    <= 1'b1;
            end else begin
                rd_o0[47:32] <= get_data(1, 0, ra[59:40]);
                rd_oe0[2]    <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[2] <= 1'b0;
            end else begin
                rd_oe0[2] <= 1'b0;
            end
        end

        /* Bank 1, subbank 1 */
        if (rc[32:31] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[63:48] <= get_data(1, 1, ra[79:60]);
                rd_oe0[3]    <= 1'b1;
            end else begin
                rd_o0[63:48] <= get_data(1, 1, ra[79:60]);
                rd_oe0[3]    <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[3] <= 1'b0;
            end else begin
                rd_oe0[3] <= 1'b0;
            end
        end

        /* Bank 2, subbank 0 */
        if (rc[41:40] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[79:64] <= get_data(2, 0, ra[99:80]);
                rd_oe0[4]    <= 1'b1;
            end else begin
                rd_o0[79:64] <= get_data(2, 0, ra[99:80]);
                rd_oe0[4]    <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[4] <= 1'b0;
            end else begin
                rd_oe0[4] <= 1'b0;
            end
        end

        /* Bank 2, subbank 1 */
        if (rc[50:49] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[95:80] <= get_data(2, 1, ra[119:100]);
                rd_oe0[5]    <= 1'b1;
            end else begin
                rd_o0[95:80] <= get_data(2, 1, ra[119:100]);
                rd_oe0[5]    <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[5] <= 1'b0;
            end else begin
                rd_oe0[5] <= 1'b0;
            end
        end

        /* Bank 3, subbank 0 */
        if (rc[59:58] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[111:96] <= get_data(3, 0, ra[139:120]);
                rd_oe0[6]     <= 1'b1;
            end else begin
                rd_o0[111:96] <= get_data(3, 0, ra[139:120]);
                rd_oe0[6]     <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[6] <= 1'b0;
            end else begin
                rd_oe0[6] <= 1'b0;
            end
        end

        /* Bank 3, subbank 1 */
        if (rc[68:67] == 2'b01) begin
            if (pipeline_reg_val) begin
                rd_o0[127:112] <= get_data(3, 1, ra[159:140]);
                rd_oe0[7]      <= 1'b1;
            end else begin
                rd_o0[127:112] <= get_data(3, 1, ra[159:140]);
                rd_oe0[7]      <= 1'b1;
            end
        end else begin
            if (pipeline_reg_val) begin
                rd_oe0[7] <= 1'b0;
            end else begin
                rd_oe0[7] <= 1'b0;
            end
        end
    end
end

initial
begin
    lclk <= 1'b1;
    mclk <= 1'b1;
    la <= 22'hZ;
    ld_oe <= 1'b0;
    lads_l <= 1'b1;
    lbe_l <= 4'hZ;
    lwrite <= 1'bZ;
    lreset_l <= 1'b0;
    fholda <= 1'b0;
    rd_oe0 <= 0;
    rd_oe1 <= 0;
    size_reg_val = 0;
    pipeline_reg_val = 0;
    #(period * 4);
    lreset_l <= 1'b1;
    #1;

    //
    // Read the information register
    //
    lads_l <= 1'b0;
    la <= 4;
    lwrite <= 1'b0;
    lblast_l <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 3);
    lblast_l <= 1'b1;
    la <= 22'hZ;

    //
    // Read the status register
    //
    lads_l <= 1'b0;
    la <= 5;
    lwrite <= 1'b0;
    lblast_l <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 3);
    lblast_l <= 1'b1;
    la <= 22'hZ;

    //
    // Enter pipelined mode
    //
    lads_l <= 1'b0;
    la <= 1;
    lwrite <= 1'b1;
    lblast_l <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 1;
    #(period * 2);
    lblast_l <= 1'b1;
    ld_oe <= 1'b0;
    la <= 22'hZ;
    pipeline_reg_val <= 1;

    //
    // Set the size register to 18 address bits
    //
    lads_l <= 1'b0;
    la <= 2;
    lwrite <= 1'b1;
    lblast_l <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 1;
    #(period * 2);
    lblast_l <= 1'b1;
    ld_oe <= 1'b0;
    la <= 22'hZ;
    size_reg_val <= 1;

    //
    // Check the mode register
    //
    lads_l <= 1'b0;
    la <= 1;
    lwrite <= 1'b0;
    lblast_l <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 3);
    lblast_l <= 1'b1;
    la <= 22'hZ;

    //
    // Write to location 0
    //
    lads_l <= 1'b0;
    la <= 2**19;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 0;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    lblast_l <= 1'b0;
    #(period * 1);
    lblast_l <= 1'b1;
    ld_oe <= 1'b0;
    la <= 22'hZ;

    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 7;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 100;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    ld_oe <= 1'b0;
    la <= 22'hZ;

    lads_l <= 1'b0;
    la <= 2**19 + 2**18;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 107;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    lblast_l <= 1'b0;
    #(period * 1);
    lblast_l <= 1'b1;
    ld_oe <= 1'b0;
    la <= 22'hZ;

    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 7;
    lwrite <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 7);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= 22'hZ;

    //
    // Test single word read at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 1;
    lwrite <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 7);
    la <= 22'hZ;

    //
    // Test single word write at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 1;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 255;
    #(period * 2);
    la <= 22'hZ;
    ld_oe <= 1'b0;

    //
    // Test double word read at bank/page boundary
    //        
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 2;
    lwrite <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 8);
    la <= la + 1;
    #(period * 1);
    la <= 22'hZ;

    //
    // Test double word write at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 2;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 254;
    #(period * 2);
    la <= la + 1;
    ld_o <= 255;
    #(period * 1);
    la <= 22'hZ;
    ld_oe <= 1'b0;

    //
    // Enter flowthrough mode
    //
    lads_l <= 1'b0;
    la <= 1;
    lwrite <= 1'b1;
    lblast_l <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 0;
    #(period * 2);
    lblast_l <= 1'b1;
    la <= 22'hZ;
    ld_oe <= 1'b0;
    pipeline_reg_val <= 0;

    //
    // Set page register to 1
    //
    lads_l <= 1'b0;
    la <= 0;
    lwrite <= 1'b1;
    lblast_l <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 1;
    #(period * 2);
    lblast_l <= 1'b1;
    la <= 22'hZ;
    ld_oe <= 1'b0;

    //
    // Write to location 0
    //
    lads_l <= 1'b0;
    la <= 2**19 + 0 * 2**18;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 0;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    lblast_l <= 1'b0;
    #(period * 1);
    lblast_l <= 1'b1;
    ld_oe <= 1'b0;
    la <= 22'hZ;

    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 7;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 200;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    ld_oe <= 1'b0;
    la <= 22'hZ;

    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 207;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= la + 1;
    ld_o <= ld_o + 1;
    lblast_l <= 1'b0;
    #(period * 1);
    lblast_l <= 1'b1;
    ld_oe <= 1'b0;
    la <= 22'hZ;

    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 7;
    lwrite <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 7);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= la + 1;
    #(period * 1);
    la <= 22'hZ;

    //
    // Test single word read at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 1;
    lwrite <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 7);
    la <= 22'hZ;

    //
    // Test single word write at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 1;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 255;
    #(period * 2);
    la <= 22'hZ;
    ld_oe <= 1'b0;

    //
    // Test double word read at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 2;
    lwrite <= 1'b0;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    #(period * 8);
    la <= la + 1;
    #(period * 1);
    la <= 22'hZ;

    //
    // Test double word write at bank/page boundary
    //         
    lads_l <= 1'b0;
    la <= 2**19 + 1 * 2**18 - 2;
    lwrite <= 1'b1;
    lbe_l <= 4'h0;
    #(period * 1);
    lads_l <= 1'b1;
    ld_oe <= 1'b1;
    ld_o <= 254;
    #(period * 2);
    la <= la + 1;
    ld_o <= ld_o + 1;
    #(period * 1);
    la <= 22'hZ;
    ld_oe <= 1'b0;
end

zbt #(
    num_bank,
    num_clock,
    addr_width,
    ctl_width,
    data_width,
    bank_group)
zbt0(
    .lclk         (lclk),
    .mclk         (mclk),
    .ramclko      (ramclk),
    .ramclki      (ramclk),
    .lreset_l     (lreset_l),
    .lwrite       (lwrite),
    .lads_l       (lads_l),
    .lblast_l     (lblast_l),
    .lbterm_l     (lbterm_l),
    .ld           (ld),
    .la           (la),
    .lready_l     (lready_l),
    .lbe_l        (lbe_l),
    .rc0          (rc[pctl_width *1-1 : pctl_width *0]),
    .ra0          (ra[paddr_width*1-1 : paddr_width*0]),
    .rd0          (rd[pdata_width*1-1 : pdata_width*0]),
    .rc1          (rc[pctl_width *2-1 : pctl_width *1]),
    .ra1          (ra[paddr_width*2-1 : paddr_width*1]),
    .rd1          (rd[pdata_width*2-1 : pdata_width*1]),
    .rc2          (rc[pctl_width *3-1 : pctl_width *2]),
    .ra2          (ra[paddr_width*3-1 : paddr_width*2]),
    .rd2          (rd[pdata_width*3-1 : pdata_width*2]),
    .rc3          (rc[pctl_width *4-1 : pctl_width *3]),
    .ra3          (ra[paddr_width*4-1 : paddr_width*3]),
    .rd3          (rd[pdata_width*4-1 : pdata_width*3]),
    .rc4          (rc[pctl_width *5-1 : pctl_width *4]),
    .ra4          (ra[paddr_width*5-1 : paddr_width*4]),
    .rd4          (rd[pdata_width*5-1 : pdata_width*4]),
    .rc5          (rc[pctl_width *6-1 : pctl_width *5]),
    .ra5          (ra[paddr_width*6-1 : paddr_width*5]),
    .rd5          (rd[pdata_width*6-1 : pdata_width*5]),
    .rc6          (rc[pctl_width *7-1 : pctl_width *6]),
    .ra6          (ra[paddr_width*7-1 : paddr_width*6]),
    .rd6          (rd[pdata_width*7-1 : pdata_width*6]),
    .rc7          (rc[pctl_width *8-1 : pctl_width *7]),
    .ra7          (ra[paddr_width*8-1 : paddr_width*7]),
    .rd7          (rd[pdata_width*8-1 : pdata_width*7]),
    .global_lbo_l (global_lbo_l),
    .flothru_l    (flothru_l),
    .fholda       (fholda));

glbl glbl();

endmodule
