/*
** zbt64-xrc2pl.v - Wrapper for zbt64_main module, targetted at
**                  ADM-XRCIIPro-Lite card.
**
** (C) Copyright Alpha Data Parallel Systems Ltd. 1999-2004
**
** Example program for:
**
**   o ADM-XRCIIPro-Lite with 2VP20 or larger
**
** This module wraps up the 'zbt64_main' module to make it compatible with
** the pinout of the ADM-XRCIIPro-Lite.
*/

`timescale 1ns / 100ps

module zbt64(
    lclk,
    mclk,
    ramclko,
    ramclki,
    lreset_l,
    lwrite,
    lads_l,
    lblast_l,
    lbterm_l,
    lad,
    lready_l,
    lbe_l,
    fholda,
    ra0,
    rc0,
    rd0,
    ra1,
    rc1,
    rd1,
    ra2,
    rc2,
    rd2,
    ra3,
    rc3,
    rd3,
    ra4,
    rc4,
    rd4,
    ra5,
    rc5,
    rd5,
    ra6,
    rc6,
    rd6,
    ra7,
    rc7,
    rd7,
    glob_lbo_l);

parameter num_bank   = 1;
parameter num_clock  = 1;
parameter addr_width = 20;
parameter ctl_width  = 15;
parameter data_width = 64;
parameter bank_group = 1;

input                               lclk;
input                               mclk;
output [num_clock-1:0]              ramclko;
input  [num_clock-1:0]              ramclki;
input                               lreset_l;
input                               lwrite;
input                               lads_l;
input                               lblast_l;
output                              lbterm_l;
inout  [63:0]                       lad;
output                              lready_l;
input  [7:0]                        lbe_l;
input                               fholda;
output [addr_width*bank_group-1:0]  ra0;
output [ctl_width *bank_group-1:0]  rc0;
inout  [data_width*bank_group-1:0]  rd0;
output [addr_width*bank_group-1:0]  ra1;
output [ctl_width *bank_group-1:0]  rc1;
inout  [data_width*bank_group-1:0]  rd1;
output [addr_width*bank_group-1:0]  ra2;
output [ctl_width *bank_group-1:0]  rc2;
inout  [data_width*bank_group-1:0]  rd2;
output [addr_width*bank_group-1:0]  ra3;
output [ctl_width *bank_group-1:0]  rc3;
inout  [data_width*bank_group-1:0]  rd3;
output [addr_width*bank_group-1:0]  ra4;
output [ctl_width *bank_group-1:0]  rc4;
inout  [data_width*bank_group-1:0]  rd4;
output [addr_width*bank_group-1:0]  ra5;
output [ctl_width *bank_group-1:0]  rc5;
inout  [data_width*bank_group-1:0]  rd5;
output [addr_width*bank_group-1:0]  ra6;
output [ctl_width *bank_group-1:0]  rc6;
inout  [data_width*bank_group-1:0]  rd6;
output [addr_width*bank_group-1:0]  ra7;
output [ctl_width *bank_group-1:0]  rc7;
inout  [data_width*bank_group-1:0]  rd7;
output                              glob_lbo_l;

wire [63:0]                         ld_o;
wire                                oe_ld;

assign lad = oe_ld ? ld_o : 64'hZZZZZZZZ;

zbt64_main #(
    num_bank,
    num_clock,
    addr_width,
    ctl_width,
    data_width,
    bank_group)
zbt64_main0(
    .lclk           (lclk),
    .mclk           (mclk),
    .ramclko        (ramclko),
    .ramclki        (ramclki),
    .lreset_l       (lreset_l),
    .lwrite         (lwrite),
    .lads_l         (lads_l),
    .lblast_l       (lblast_l),
    .lbterm_l       (lbterm_l),
    .la_i           (lad[23:3]),
    .ld_i           (lad),
    .ld_o           (ld_o),
    .oe_ld          (oe_ld),
    .lready_l       (lready_l),
    .lbe_l          (lbe_l),
    .fholda         (fholda),
    .ra0            (ra0),
    .rc0            (rc0),
    .rd0            (rd0),
    .ra1            (ra1),
    .rc1            (rc1),
    .rd1            (rd1),
    .ra2            (ra2),
    .rc2            (rc2),
    .rd2            (rd2),
    .ra3            (ra3),
    .rc3            (rc3),
    .rd3            (rd3),
    .ra4            (ra4),
    .rc4            (rc4),
    .rd4            (rd4),
    .ra5            (ra5),
    .rc5            (rc5),
    .rd5            (rd5),
    .ra6            (ra6),
    .rc6            (rc6),
    .rd6            (rd6),
    .ra7            (ra7),
    .rc7            (rc7),
    .rd7            (rd7),
    .glob_lbo_l     (glob_lbo_l),
    .glob_flothru_l ());

endmodule
