--
-- clocks_dll.vhd - clock multiplication module using DLLs for
--                  Virtex/Virtex-E/Virtex-EM
--
-- (c) Alpha Data Parallel Systems Ltd. 2001
--
-- The local bus clock is multiplied by 2 using a Virtex DLL.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity clocks is
   port(
      rst              : in    std_logic;
      lclk             : in    std_logic;
      mclk             : in    std_logic;
      clk              : out   std_logic;
      multclk          : out   std_logic;
      locked           : out   std_logic_vector(31 downto 0));
end clocks;

architecture DLL of clocks is

    signal lclk_i:       std_logic;
    signal clk0:         std_logic;
    signal i_clk:        std_logic;
    signal multclk0:     std_logic;
    
    signal logic0:       std_logic;
    signal logic1:       std_logic;

    component IBUFG
        port(
            I:             in       std_logic;
            O:             out      std_logic);
    end component;
    
    component BUFG
        port(
            I:             in       std_logic;
            O:             out      std_logic);
    end component;
    
    component CLKDLL 
        port(
            CLKIN, CLKFB, RST                                 : in  std_logic; 
            CLK0, CLK90, CLK180, CLK270, CLK2X, CLKDV, LOCKED : out std_logic);
    end component;
    
begin

    logic0 <= '0';
    logic1 <= '1';

    clk <= i_clk;
    
    --
    -- Input LCLK
    --

    ibufg0: IBUFG
        port map(
            I => lclk,
            O => lclk_i);
    
    --
    -- Generate clocks locked to LCLK and LCLK x 2
    --

    dll0: CLKDLL
        port map(
            CLK0   => clk0,
            CLK2X  => multclk0,
            LOCKED => locked(0),
            CLKIN  => lclk_i,
            CLKFB  => i_clk,
            RST    => logic0);

    locked(31 downto 1) <= (others => '0');

    bufg0: BUFG
        port map(
            I => clk0,
            O => i_clk);
    
    bufg1: BUFG
        port map(
            I => multclk0,
            O => multclk);

end DLL;
