--
-- frontio.vhd - FPGA design to output a walking 1 on I/O pins
--
-- (C) Copyright Alpha Data Parallel Systems Ltd. 1999-2004
--
-- Example FPGA design for:
--
--   o ADM-XRC
--   o ADM-XRCII-Lite
--   o ADM-XRCII
--
-- This example outputs a walking bit on the 34 user I/O pins at
-- the front panel connector.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_arith.all;

entity frontio is
    port(
        lclk:           in        std_logic;
        lreset_l:       in        std_logic;
        frontio:        out       std_logic_vector(33 downto 0));
end frontio;

architecture rtl of frontio is

    signal rst:        std_logic;
    signal i_frontio:  std_logic_vector(33 downto 0);

begin

    frontio <= i_frontio;
    rst <= not lreset_l;

    --
    -- Shift pattern up from 0->33
    --
    walk: process(lclk, rst)
    begin
        if rst = '1' then
            i_frontio <= EXT("1", 34);
        elsif lclk'event and lclk = '1' then
            i_frontio <= i_frontio(32 downto 0) & i_frontio(33);
        end if;
    end process;

end rtl;
