--
-- test_memory-xpi.vhd - Testbench for entity 'memory', targetting ADP-XPI
--
-- (C) Copyright Alpha Data 2008
--
-- NOT SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library std;
use std.textio.all;

library plxsim;
use plxsim.plxsim.all;

library work;
use work.memif.all;
use work.memif_sim.all;

entity test_memory is
    generic(
        test_id : in natural);
end test_memory;

architecture behav of test_memory is

    constant registered_dimms : boolean := true;
    constant refclk_period : time := 8.0 ns;
    constant lclk_period : time := 15 ns;
    constant mclk_period : time := lclk_period / 2;

    constant num_agent : natural := 1;

    constant priorities : integer_vector_t(0 to num_agent - 1) := (others => 0);

    constant bank0 : bank_t := (enable => true, ra_width => 19, rc_width => 31, rd_width => 64);
    constant bank1 : bank_t := (enable => true, ra_width => 19, rc_width => 31, rd_width => 64);
    constant bank2 : bank_t := (enable => true, ra_width => 19, rc_width => 31, rd_width => 64);
    constant bank3 : bank_t := (enable => true, ra_width => 19, rc_width => 31, rd_width => 64);
    constant bank4 : bank_t := (enable => true, ra_width => 22, rc_width => 11, rd_width => 32);
    constant bank5 : bank_t := no_bank;
    constant bank6 : bank_t := no_bank;
    constant bank7 : bank_t := no_bank;
    constant bank8 : bank_t := no_bank;
    constant bank9 : bank_t := no_bank;
    constant bank10 : bank_t := no_bank;
    constant bank11 : bank_t := no_bank;
    constant bank12 : bank_t := no_bank;
    constant bank13 : bank_t := no_bank;
    constant bank14 : bank_t := no_bank;
    constant bank15 : bank_t := no_bank;
    constant num_ramclk : natural := 1;

    signal lclk : std_logic := '1';
    signal lreset_l : std_logic := '0';
    signal lwrite : std_logic := 'Z';
    signal lads_l : std_logic := 'Z';
    signal l64_l : std_logic := 'Z';
    signal lblast_l : std_logic := 'Z';
    signal lbterm_l : std_logic := 'Z';
    signal lad : std_logic_vector(31 downto 0) := (others => 'Z');
    signal lready_l : std_logic := 'Z';
    signal lbe_l : std_logic_vector(3 downto 0) := (others => 'Z');
    signal fholda : std_logic := '0';
    signal lhold : std_logic_vector(num_agent - 1 downto 0);
    signal lholda : std_logic_vector(num_agent - 1 downto 0);
    signal mclk : std_logic := '1';
    signal refclk_p : std_logic := '1';
    signal refclk_n : std_logic := '0';
    signal ra0 : std_logic_vector(bank0.ra_width - 1 downto 0) := (others => 'Z');
    signal rc0 : std_logic_vector(bank0.rc_width - 1 downto 0) := (others => 'Z');
    signal rd0 : std_logic_vector(bank0.rd_width - 1 downto 0) := (others => 'Z');
    signal ra1 : std_logic_vector(bank1.ra_width - 1 downto 0) := (others => 'Z');
    signal rc1 : std_logic_vector(bank1.rc_width - 1 downto 0) := (others => 'Z');
    signal rd1 : std_logic_vector(bank1.rd_width - 1 downto 0) := (others => 'Z');
    signal ra2 : std_logic_vector(bank2.ra_width - 1 downto 0) := (others => 'Z');
    signal rc2 : std_logic_vector(bank2.rc_width - 1 downto 0) := (others => 'Z');
    signal rd2 : std_logic_vector(bank2.rd_width - 1 downto 0) := (others => 'Z');
    signal ra3 : std_logic_vector(bank3.ra_width - 1 downto 0) := (others => 'Z');
    signal rc3 : std_logic_vector(bank3.rc_width - 1 downto 0) := (others => 'Z');
    signal rd3 : std_logic_vector(bank3.rd_width - 1 downto 0) := (others => 'Z');
    signal ra4 : std_logic_vector(bank4.ra_width - 1 downto 0) := (others => 'Z');
    signal rc4 : std_logic_vector(bank4.rc_width - 1 downto 0) := (others => 'Z');
    signal rd4 : std_logic_vector(bank4.rd_width - 1 downto 0) := (others => 'Z');
    signal ra5 : std_logic_vector(bank5.ra_width - 1 downto 0) := (others => 'Z');
    signal rc5 : std_logic_vector(bank5.rc_width - 1 downto 0) := (others => 'Z');
    signal rd5 : std_logic_vector(bank5.rd_width - 1 downto 0) := (others => 'Z');
    signal ra6 : std_logic_vector(bank6.ra_width - 1 downto 0) := (others => 'Z');
    signal rc6 : std_logic_vector(bank6.rc_width - 1 downto 0) := (others => 'Z');
    signal rd6 : std_logic_vector(bank6.rd_width - 1 downto 0) := (others => 'Z');
    signal ra7 : std_logic_vector(bank7.ra_width - 1 downto 0) := (others => 'Z');
    signal rc7 : std_logic_vector(bank7.rc_width - 1 downto 0) := (others => 'Z');
    signal rd7 : std_logic_vector(bank7.rd_width - 1 downto 0) := (others => 'Z');
    signal ra8 : std_logic_vector(bank8.ra_width - 1 downto 0) := (others => 'Z');
    signal rc8 : std_logic_vector(bank8.rc_width - 1 downto 0) := (others => 'Z');
    signal rd8 : std_logic_vector(bank8.rd_width - 1 downto 0) := (others => 'Z');
    signal ra9 : std_logic_vector(bank9.ra_width - 1 downto 0) := (others => 'Z');
    signal rc9 : std_logic_vector(bank9.rc_width - 1 downto 0) := (others => 'Z');
    signal rd9 : std_logic_vector(bank9.rd_width - 1 downto 0) := (others => 'Z');
    signal ra10 : std_logic_vector(bank10.ra_width - 1 downto 0) := (others => 'Z');
    signal rc10 : std_logic_vector(bank10.rc_width - 1 downto 0) := (others => 'Z');
    signal rd10 : std_logic_vector(bank10.rd_width - 1 downto 0) := (others => 'Z');
    signal ra11 : std_logic_vector(bank11.ra_width - 1 downto 0) := (others => 'Z');
    signal rc11 : std_logic_vector(bank11.rc_width - 1 downto 0) := (others => 'Z');
    signal rd11 : std_logic_vector(bank11.rd_width - 1 downto 0) := (others => 'Z');
    signal ra12 : std_logic_vector(bank12.ra_width - 1 downto 0) := (others => 'Z');
    signal rc12 : std_logic_vector(bank12.rc_width - 1 downto 0) := (others => 'Z');
    signal rd12 : std_logic_vector(bank12.rd_width - 1 downto 0) := (others => 'Z');
    signal ra13 : std_logic_vector(bank13.ra_width - 1 downto 0) := (others => 'Z');
    signal rc13 : std_logic_vector(bank13.rc_width - 1 downto 0) := (others => 'Z');
    signal rd13 : std_logic_vector(bank13.rd_width - 1 downto 0) := (others => 'Z');
    signal ra14 : std_logic_vector(bank14.ra_width - 1 downto 0) := (others => 'Z');
    signal rc14 : std_logic_vector(bank14.rc_width - 1 downto 0) := (others => 'Z');
    signal rd14 : std_logic_vector(bank14.rd_width - 1 downto 0) := (others => 'Z');
    signal ra15 : std_logic_vector(bank15.ra_width - 1 downto 0) := (others => 'Z');
    signal rc15 : std_logic_vector(bank15.rc_width - 1 downto 0) := (others => 'Z');
    signal rd15 : std_logic_vector(bank15.rd_width - 1 downto 0) := (others => 'Z');
    signal ramclk : std_logic_vector(num_ramclk - 1 downto 0);

    signal ra0_ram : std_logic_vector(bank0.ra_width - 1 downto 0) := (others => 'Z');
    signal rc0_ram : std_logic_vector(bank0.rc_width - 1 downto 0) := (others => 'Z');
    signal rd0_ram : std_logic_vector(bank0.rd_width - 1 downto 0) := (others => 'Z');
    signal ra1_ram : std_logic_vector(bank1.ra_width - 1 downto 0) := (others => 'Z');
    signal rc1_ram : std_logic_vector(bank1.rc_width - 1 downto 0) := (others => 'Z');
    signal rd1_ram : std_logic_vector(bank1.rd_width - 1 downto 0) := (others => 'Z');
    signal ra2_ram : std_logic_vector(bank2.ra_width - 1 downto 0) := (others => 'Z');
    signal rc2_ram : std_logic_vector(bank2.rc_width - 1 downto 0) := (others => 'Z');
    signal rd2_ram : std_logic_vector(bank2.rd_width - 1 downto 0) := (others => 'Z');
    signal ra3_ram : std_logic_vector(bank3.ra_width - 1 downto 0) := (others => 'Z');
    signal rc3_ram : std_logic_vector(bank3.rc_width - 1 downto 0) := (others => 'Z');
    signal rd3_ram : std_logic_vector(bank3.rd_width - 1 downto 0) := (others => 'Z');
    signal ra4_ram : std_logic_vector(bank4.ra_width - 1 downto 0) := (others => 'Z');
    signal rc4_ram : std_logic_vector(bank4.rc_width - 1 downto 0) := (others => 'Z');
    signal rd4_ram : std_logic_vector(bank4.rd_width - 1 downto 0) := (others => 'Z');
    signal cq_ram : std_logic_vector(4 downto 4);
    signal cq_ram_l : std_logic_vector(4 downto 4);

    signal sa0 : std_logic_vector(2 downto 0);
    signal sa1 : std_logic_vector(2 downto 0);
    signal sa2 : std_logic_vector(2 downto 0);
    signal sa3 : std_logic_vector(2 downto 0);

    signal scl : std_logic;
    signal sda : std_logic;

    signal bus_in : locbus_in_t;
    signal bus_out : locbus_out_t := init_locbus_out;

    signal logic0, logic1 : std_logic;

    procedure report_message(
        str : in string)
    is
        constant colon : string(1 to 2) := ": ";
        variable msg : line;
    begin
        write(msg, now);
        write(msg, colon);
        write(msg, str);
        writeline(output, msg);
    end;

    procedure wait_cycles(
        constant n : in natural;
        signal   k : in std_logic)
    is begin
        for i in 0 to n - 1 loop
            wait until k'event and k = '1';
        end loop;
	wait for 1 ns;
    end;

    function max(
        a, b : in integer)
    return integer is
    begin
        if a > b then
            return a;
        else
            return b;
        end if;
    end;

    function min(
        a, b : in integer)
    return integer is
    begin
        if a < b then
            return a;
        else
            return b;
        end if;
    end;

    component memory
        generic(
            bank0             : in    bank_t := (enable => true, ra_width => 22, rc_width => 11, rd_width => 32);
            bank1             : in    bank_t := (enable => true, ra_width => 22, rc_width => 11, rd_width => 32);
            bank2             : in    bank_t := (enable => true, ra_width => 22, rc_width => 11, rd_width => 32);
            bank3             : in    bank_t := (enable => true, ra_width => 22, rc_width => 11, rd_width => 32);
            bank4             : in    bank_t := (enable => true, ra_width => 15, rc_width => 15, rd_width => 32);
            bank5             : in    bank_t := (enable => true, ra_width => 15, rc_width => 15, rd_width => 32);
            bank6             : in    bank_t := no_bank;
            bank7             : in    bank_t := no_bank;
            bank8             : in    bank_t := no_bank;
            bank9             : in    bank_t := no_bank;
            bank10            : in    bank_t := no_bank;
            bank11            : in    bank_t := no_bank;
            bank12            : in    bank_t := no_bank;
            bank13            : in    bank_t := no_bank;
            bank14            : in    bank_t := no_bank;
            bank15            : in    bank_t := no_bank;
            num_ramclk        : in    natural := 1);
        port(
            -- Local bus
            lclk              : in    std_logic;
            lreset_l          : in    std_logic; 
            lwrite            : in    std_logic;
            lads_l            : in    std_logic;
            lblast_l          : in    std_logic;
            lbe_l             : in    std_logic_vector(3 downto 0);
            lad               : inout std_logic_vector(31 downto 0);
            lbterm_l          : inout std_logic;
            lready_l          : inout std_logic; 
            fholda            : in    std_logic;
            -- General purpose clock
            mclk              : in    std_logic;
            -- Reference clock
            refclk_p          : in    std_logic;
            refclk_n          : in    std_logic;
            -- Memory banks (up to 16 supported by this design)
            ra0               : out   std_logic_vector(bank0.ra_width - 1 downto 0);
            rc0               : inout std_logic_vector(bank0.rc_width - 1 downto 0);
            rd0               : inout std_logic_vector(bank0.rd_width - 1 downto 0);
            ra1               : out   std_logic_vector(bank1.ra_width - 1 downto 0);
            rc1               : inout std_logic_vector(bank1.rc_width - 1 downto 0);
            rd1               : inout std_logic_vector(bank1.rd_width - 1 downto 0);
            ra2               : out   std_logic_vector(bank2.ra_width - 1 downto 0);
            rc2               : inout std_logic_vector(bank2.rc_width - 1 downto 0);
            rd2               : inout std_logic_vector(bank2.rd_width - 1 downto 0);
            ra3               : out   std_logic_vector(bank3.ra_width - 1 downto 0);
            rc3               : inout std_logic_vector(bank3.rc_width - 1 downto 0);
            rd3               : inout std_logic_vector(bank3.rd_width - 1 downto 0);
            ra4               : out   std_logic_vector(bank4.ra_width - 1 downto 0);
            rc4               : inout std_logic_vector(bank4.rc_width - 1 downto 0);
            rd4               : inout std_logic_vector(bank4.rd_width - 1 downto 0);
            ra5               : out   std_logic_vector(bank5.ra_width - 1 downto 0);
            rc5               : inout std_logic_vector(bank5.rc_width - 1 downto 0);
            rd5               : inout std_logic_vector(bank5.rd_width - 1 downto 0);
            ra6               : out   std_logic_vector(bank6.ra_width - 1 downto 0);
            rc6               : inout std_logic_vector(bank6.rc_width - 1 downto 0);
            rd6               : inout std_logic_vector(bank6.rd_width - 1 downto 0);
            ra7               : out   std_logic_vector(bank7.ra_width - 1 downto 0);
            rc7               : inout std_logic_vector(bank7.rc_width - 1 downto 0);
            rd7               : inout std_logic_vector(bank7.rd_width - 1 downto 0);
            ra8               : out   std_logic_vector(bank8.ra_width - 1 downto 0);
            rc8               : inout std_logic_vector(bank8.rc_width - 1 downto 0);
            rd8               : inout std_logic_vector(bank8.rd_width - 1 downto 0);
            ra9               : out   std_logic_vector(bank9.ra_width - 1 downto 0);
            rc9               : inout std_logic_vector(bank9.rc_width - 1 downto 0);
            rd9               : inout std_logic_vector(bank9.rd_width - 1 downto 0);
            ra10              : out   std_logic_vector(bank10.ra_width - 1 downto 0);
            rc10              : inout std_logic_vector(bank10.rc_width - 1 downto 0);
            rd10              : inout std_logic_vector(bank10.rd_width - 1 downto 0);
            ra11              : out   std_logic_vector(bank11.ra_width - 1 downto 0);
            rc11              : inout std_logic_vector(bank11.rc_width - 1 downto 0);
            rd11              : inout std_logic_vector(bank11.rd_width - 1 downto 0);
            ra12              : out   std_logic_vector(bank12.ra_width - 1 downto 0);
            rc12              : inout std_logic_vector(bank12.rc_width - 1 downto 0);
            rd12              : inout std_logic_vector(bank12.rd_width - 1 downto 0);
            ra13              : out   std_logic_vector(bank13.ra_width - 1 downto 0);
            rc13              : inout std_logic_vector(bank13.rc_width - 1 downto 0);
            rd13              : inout std_logic_vector(bank13.rd_width - 1 downto 0);
            ra14              : out   std_logic_vector(bank14.ra_width - 1 downto 0);
            rc14              : inout std_logic_vector(bank14.rc_width - 1 downto 0);
            rd14              : inout std_logic_vector(bank14.rd_width - 1 downto 0);
            ra15              : out   std_logic_vector(bank15.ra_width - 1 downto 0);
            rc15              : inout std_logic_vector(bank15.rc_width - 1 downto 0);
            rd15              : inout std_logic_vector(bank15.rd_width - 1 downto 0);
            ramclko           : out   std_logic_vector(num_ramclk - 1 downto 0);
            scl               : inout std_logic;
            sda               : inout std_logic);
    end component;

    component MT46V16M16 is
        generic(                                   -- Timing for -75Z CL2
            tCK       : TIME    :=  7.500 ns;
            tCH       : TIME    :=  3.375 ns;       -- 0.45*tCK
            tCL       : TIME    :=  3.375 ns;       -- 0.45*tCK
            tDH       : TIME    :=  0.500 ns;
            tDS       : TIME    :=  0.500 ns;
            tIH       : TIME    :=  0.900 ns;
            tIS       : TIME    :=  0.900 ns;
            tMRD      : TIME    := 15.000 ns;
            tRAS      : TIME    := 40.000 ns;
            tRAP      : TIME    := 20.000 ns;
            tRC       : TIME    := 65.000 ns;
            tRFC      : TIME    := 75.000 ns;
            tRCD      : TIME    := 20.000 ns;
            tRP       : TIME    := 20.000 ns;
            tRRD      : TIME    := 15.000 ns;
            tWR       : TIME    := 15.000 ns;
            addr_bits : INTEGER := 13;
            data_bits : INTEGER := 16;
            cols_bits : INTEGER :=  9);
        port(
            Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
            Dqs   : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";
            Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
            Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
            Clk   : IN    STD_LOGIC;
            Clk_n : IN    STD_LOGIC;
            Cke   : IN    STD_LOGIC;
            Cs_n  : IN    STD_LOGIC;
            Ras_n : IN    STD_LOGIC;
            Cas_n : IN    STD_LOGIC;
            We_n  : IN    STD_LOGIC;
            Dm    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0));
    end component;

    procedure check_status_register(
        signal   bus_in  : in    locbus_in_t;
        signal   bus_out : out   locbus_out_t)
    is
        constant be : byte_enable_t(0 to 3) := X"F";
        variable data : byte_vector_t(0 to 3);
        variable addr : std_logic_vector(31 downto 0);
        variable val32 : std_logic_vector(31 downto 0);
        variable n : natural;
    begin
        plxsim_read(2, true, X"00000010", be, data, n, bus_in, bus_out);
        val32 := conv_std_logic_vector(data);
        if val32(0) /= '1' then
            report_message("*** STATUS register reports LCLK DCM not locked");
        end if;
    end;

    procedure configure_memory_subsystem(
        signal   bus_in  : in    locbus_in_t;
        signal   bus_out : out   locbus_out_t)
    is
        constant be : byte_enable_t(0 to 3) := X"F";
        variable data : byte_vector_t(0 to 3);
        variable addr : std_logic_vector(31 downto 0);
        variable val32 : std_logic_vector(31 downto 0);
        variable n : natural;
    begin
        for i in 0 to 4 loop
            report_message("Configuring bank " & conv_string(i));

            -- Select bank 'i'
            data := conv_byte_vector(CONV_STD_LOGIC_VECTOR(i, 32));
            plxsim_write(2, true, X"00000000", be, data, n, bus_in, bus_out);

            -- Set operating mode for bank 'i'
            addr := X"00000040";
            addr := addr + i * 4;
            case i is
                when 0 | 1 | 2 | 3 =>
                    if registered_dimms then
                        -- Set bank mode (DDR SDRAM) to registered, X8/X16, row bits 13, col bits 9, bank bits 2, physical banks 2
                        data(0 to 3) := conv_byte_vector(X"00000185");
                    else
                        -- Set bank mode (DDR SDRAM) to unbuffered, X8/X16, row bits 13, col bits 9, bank bits 2, physical banks 2
                        data(0 to 3) := conv_byte_vector(X"00000184");
                    end if;

                when 4 =>
                    -- Set bank mode (DDR2 SRAM) to BL4, DLL enabled
                    data(0 to 3) := conv_byte_vector(X"00000001");

                when others =>
                    null;
            end case;
            plxsim_write(2, true, addr, be(0 to 3), data(0 to 3), n, bus_in, bus_out);
        end loop;
    end;

    procedure reset_memory_subsystem(
        signal   bus_in  : in    locbus_in_t;
        signal   bus_out : out   locbus_out_t)
    is
        constant be : byte_enable_t(0 to 3) := X"F";
        variable data : byte_vector_t(0 to 3);
        variable val32 : std_logic_vector(31 downto 0);
        variable n : natural;
    begin
        -- Write 0x1 to MEMCTL register to assert reset
        val32 := X"00000001";
        data := conv_byte_vector(val32);
        plxsim_write(2, true, X"00000008", be, data, n, bus_in, bus_out);
        val32 := conv_std_logic_vector(data);
        -- Write 0x0 to MEMCTL register to deassert reset
        val32 := X"00000000";
        data := conv_byte_vector(val32);
        plxsim_write(2, true, X"00000008", be, data, n, bus_in, bus_out);
        val32 := conv_std_logic_vector(data);
    end;

    procedure wait_until_trained(
        constant mask    : in    std_logic_vector(15 downto 0);
        signal   bus_in  : in    locbus_in_t;
        signal   bus_out : out   locbus_out_t)
    is
        constant be : byte_enable_t(0 to 3) := X"F";
        variable data : byte_vector_t(0 to 3);
        variable val32 : std_logic_vector(31 downto 0);
        variable n : natural;
    begin
        loop
            -- Read MEMSTAT register (for training flags)
            plxsim_read(2, true, X"00000018", be, data, n, bus_in, bus_out);
            val32 := conv_std_logic_vector(data);
            exit when (val32(15 downto 0) and mask) = mask; -- test for all ports trained
        end loop;    
    end;

    procedure test_host_access(
        signal bus_in  : in    locbus_in_t;
        signal bus_out : out   locbus_out_t)
    is
        constant len : natural := 10000;
        variable addr : std_logic_vector(31 downto 0);
        variable dummy32 : byte_vector_t(0 to 3);
        variable data, chk : byte_vector_t(0 to len - 1);
        variable be : byte_enable_t(0 to len - 1);
        variable n : natural;
        variable max : natural;
        variable remaining : natural;
        variable pos : natural;
        variable chunk : natural;
    begin
        -- Wait for DCMs to achieve lock and for memory banks to be trained
        report_message("Waiting 50 us to allow LCLK DCM to achieve lock");
        wait for 50 us;
        wait_cycles(1, lclk);

        be := (others => '1');

        plxsim_request_bus(true, bus_in, bus_out);

        -- Check that STATUS register indicates that LCLK DCM has achieved lock
        check_status_register(bus_in, bus_out);

        -- Set the operating parameters for the memory ports
        report_message("Configuring memory port operating parameters");
        configure_memory_subsystem(bus_in, bus_out);

        -- Reset the memory subsystem
        report_message("Resetting memory subsystem");
        reset_memory_subsystem(bus_in, bus_out);

        -- Wait until all memory ports have been trained
        report_message("Waiting for completion of memory port training");
        wait_until_trained(X"001F", bus_in, bus_out);

        for i in 0 to 4 loop
            -- Select bank 'i'
            data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(i, 32));
            plxsim_write(2, true, X"00000000", be(0 to 3), data(0 to 3), n, bus_in, bus_out);

            -- Write some data
            report_message("Writing data to bank " & conv_string(i));
            for i in 0 to len / 2 - 1 loop
                data(i * 2 to i * 2 + 1) := conv_byte_vector(conv_std_logic_vector(i, 16));
            end loop;
            pos := 0;
            remaining := len;
            max := 100;
            addr := X"00200000";
            while pos < len loop
                chunk := min(remaining, max);
                plxsim_write(2, true, addr, be(pos to pos + chunk - 1), data(pos to pos + chunk - 1), n, bus_in, bus_out);
                max := max - 4;
                if max = 0 then
                    max := 100;
                end if;
                pos := pos + n;
                remaining := remaining - n;
                addr := addr + n;
            end loop;

            -- Read some data
            report_message("Reading back bank " & conv_string(i));
            chk := (others => (others => '0'));
            pos := 0;
            remaining := len;
            max := 100;
            addr := X"00200000";
            while pos < len loop
                chunk := min(remaining, max);
                plxsim_read(2, true, addr, be(pos to pos + chunk - 1), chk(pos to pos + chunk - 1), n, bus_in, bus_out);
                max := max - 4;
                if max = 0 then
                    max := 100;
                end if;
                for i in pos to pos + n - 1 loop
                    if chk(i) /= data(i) then
                        report_message("*** " & conv_string(now) & " index=" & conv_string_hex(i) &
                                       " exp=0x" & conv_string_hex(data(i)) &
                                       " act=0x" & conv_string_hex(chk(i)));
                    end if;
                end loop;
                pos := pos + n;
                remaining := remaining - n;
                addr := addr + n;
            end loop;
        end loop;

        plxsim_request_bus(false, bus_in, bus_out);

	assert false report "Simulation of host memory access (ADP-XPI) complete" severity failure;
        wait;
    end;

    procedure on_chip_memory_test(
        signal bus_in  : in    locbus_in_t;
        signal bus_out : out   locbus_out_t)
    is
        variable addr : std_logic_vector(31 downto 0);
        variable val32 : std_logic_vector(31 downto 0);
        variable data, chk : byte_vector_t(0 to 31);
        variable be : byte_enable_t(0 to 31);
        variable n : natural;
    begin
        -- Wait for DCMs to achieve lock
        report_message("Waiting 50 us to allow LCLK DCM to achieve lock");
        wait for 50 us;
        wait_cycles(1, lclk);

        be := (others => '1');

        plxsim_request_bus(true, bus_in, bus_out);

        -- Read STATUS register (for DCM lock flags)
        plxsim_read(2, true, X"00000010", be(0 to 3), data(0 to 3), n, bus_in, bus_out);

        -- Set the operating parameters for the memory ports
        report_message("Configuring memory port operating parameters");
        configure_memory_subsystem(bus_in, bus_out);

        -- Reset the memory subsystem
        report_message("Resetting memory subsystem");
        reset_memory_subsystem(bus_in, bus_out);

        -- Wait until all memory ports have been trained
        report_message("Waiting for completion of memory port training");
        wait_until_trained(X"001F", bus_in, bus_out);

        -- Kick off memory tests on banks 0 to 4
        report_message("Initiating on-chip memory test");
        data(0 to 3) := conv_byte_vector(X"0000001F");
        plxsim_write(2, true, X"000001fc", be(0 to 3), data(0 to 3), n, bus_in, bus_out);
        wait_cycles(32, lclk);

        -- Select bank 1
        data(0 to 3) := conv_byte_vector(X"00000001");
        plxsim_write(2, true, X"00000000", be(0 to 3), data(0 to 3), n, bus_in, bus_out);

        -- Wait for memory tests to complete
        while true loop
            -- Read some data from memory
            plxsim_read(2, true, X"00200400", be(0 to 31), chk(0 to 31), n, bus_in, bus_out);

            -- Read memory test status in USER63 register
            plxsim_read(2, true, X"000001fc", be(0 to 3), data(0 to 3), n, bus_in, bus_out);
            val32 := conv_std_logic_vector(data(0 to 3));
            exit when val32(15 downto 0) = X"FFFF";

            wait_cycles(20, lclk);
        end loop;

        report_message("On-chip memory test completed, ERROR=0x" & conv_string_hex(val32(31 downto 16)));

        plxsim_request_bus(false, bus_in, bus_out);

	assert false report "Simulation of on-chip memory test (ADP-XPI) complete" severity failure;
        wait;
    end;

begin

    logic0 <= '0';
    logic1 <= '1';

    refclk_p <= not refclk_p after refclk_period / 2;
    refclk_n <= not refclk_n after refclk_period / 2;

    lclk <= not lclk after lclk_period / 2;

    mclk <= not mclk after mclk_period / 2;

    lreset_l <= transport '1' after 50 ns;

    lads_l <= 'H';
    l64_l <= 'H';
    lblast_l <= 'H';
    lbterm_l <= 'H';
    lready_l <= 'H';
    lwrite <= 'H';

    scl <= 'H';
    sda <= 'H';

    sa0 <= "000";
    sa1 <= "001";
    sa2 <= "010";
    sa3 <= "011";

    main : process
    begin
        case test_id is
            when 0 =>
                test_host_access(bus_in, bus_out);
            when 1 =>
                on_chip_memory_test(bus_in, bus_out);
            when others =>
                assert false report "Invalid test_id value : " & conv_string(test_id) severity failure;
        end case;
        wait;
    end process;

    trace_model_0 : ddrsdram_dimm_trace_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank0.ra_width - 3,
            max_phys_bank => 4,
            dq_width => bank0.rd_width,
            registered => registered_dimms,
            t_trace_k => 0.5 ns,
            t_trace_ctl => 0.5 ns,
            t_trace_dm => 0.5 ns,
            t_trace_dq => 0.5 ns,
            t_trace_dqs => 0.5 ns)
        port map(
            mk0 => rc0(16),
            mk0_l => rc0(19),
            mk1 => rc0(17),
            mk1_l => rc0(20),
            mk2 => rc0(18),
            mk2_l => rc0(21),
            mcke => rc0(23 downto 22),
            mwe_l => rc0(24),
            mcas_l => rc0(25),
            mras_l => rc0(26),
            mcs_l => rc0(30 downto 27),
            mba => ra0(18 downto 16),
            ma => ra0(15 downto 0),
            mdq => rd0,
            mdm => rc0(7 downto 0),
            mdqs => rc0(15 downto 8),
            dk0 => rc0_ram(16),
            dk0_l => rc0_ram(19),
            dk1 => rc0_ram(17),
            dk1_l => rc0_ram(20),
            dk2 => rc0_ram(18),
            dk2_l => rc0_ram(21),
            dcke => rc0_ram(23 downto 22),
            dwe_l => rc0_ram(24),
            dcas_l => rc0_ram(25),
            dras_l => rc0_ram(26),
            dcs_l => rc0_ram(30 downto 27),
            dba => ra0_ram(18 downto 16),
            da => ra0_ram(15 downto 0),
            ddq => rd0_ram,
            ddm => rc0_ram(7 downto 0),
            ddqs => rc0_ram(15 downto 8));

    mem_model_0 : ddrsdram_dimm_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank0.ra_width - 3,
            max_phys_bank => 4,
            num_phys_bank => 1,
            dq_width => bank0.rd_width,
            registered => registered_dimms,
            extra_dqs => false)
        port map(
            k0 => rc0_ram(16),
            k0_l => rc0_ram(19),
            k1 => rc0_ram(17),
            k1_l => rc0_ram(20),
            k2 => rc0_ram(18),
            k2_l => rc0_ram(21),
            cke => rc0_ram(23 downto 22),
            we_l => rc0_ram(24),
            cas_l => rc0_ram(25),
            ras_l => rc0_ram(26),
            cs_l => rc0_ram(30 downto 27),
            ba => ra0_ram(18 downto 16),
            a => ra0_ram(15 downto 0),
            dq => rd0_ram,
            dm => rc0_ram(7 downto 0),
            dqs => rc0_ram(15 downto 8),
            sa => sa0,
            scl => scl,
            sda => sda);

    trace_model_1 : ddrsdram_dimm_trace_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank1.ra_width - 3,
            max_phys_bank => 4,
            dq_width => bank0.rd_width,
            registered => registered_dimms,
            t_trace_k => 0.5 ns,
            t_trace_ctl => 0.5 ns,
            t_trace_dm => 0.5 ns,
            t_trace_dq => 0.5 ns,
            t_trace_dqs => 0.5 ns)
        port map(
            mk0 => rc1(16),
            mk0_l => rc1(19),
            mk1 => rc1(17),
            mk1_l => rc1(20),
            mk2 => rc1(18),
            mk2_l => rc1(21),
            mcke => rc1(23 downto 22),
            mwe_l => rc1(24),
            mcas_l => rc1(25),
            mras_l => rc1(26),
            mcs_l => rc1(30 downto 27),
            mba => ra1(18 downto 16),
            ma => ra1(15 downto 0),
            mdq => rd1,
            mdm => rc1(7 downto 0),
            mdqs => rc1(15 downto 8),
            dk0 => rc1_ram(16),
            dk0_l => rc1_ram(19),
            dk1 => rc1_ram(17),
            dk1_l => rc1_ram(20),
            dk2 => rc1_ram(18),
            dk2_l => rc1_ram(21),
            dcke => rc1_ram(23 downto 22),
            dwe_l => rc1_ram(24),
            dcas_l => rc1_ram(25),
            dras_l => rc1_ram(26),
            dcs_l => rc1_ram(30 downto 27),
            dba => ra1_ram(18 downto 16),
            da => ra1_ram(15 downto 0),
            ddq => rd1_ram,
            ddm => rc1_ram(7 downto 0),
            ddqs => rc1_ram(15 downto 8));

    mem_model_1 : ddrsdram_dimm_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank1.ra_width - 3,
            max_phys_bank => 4,
            num_phys_bank => 1,
            dq_width => bank1.rd_width,
            registered => registered_dimms,
            extra_dqs => false)
        port map(
            k0 => rc1_ram(16),
            k0_l => rc1_ram(19),
            k1 => rc1_ram(17),
            k1_l => rc1_ram(20),
            k2 => rc1_ram(18),
            k2_l => rc1_ram(21),
            cke => rc1_ram(23 downto 22),
            we_l => rc1_ram(24),
            cas_l => rc1_ram(25),
            ras_l => rc1_ram(26),
            cs_l => rc1_ram(30 downto 27),
            ba => ra1_ram(18 downto 16),
            a => ra1_ram(15 downto 0),
            dq => rd1_ram,
            dm => rc1_ram(7 downto 0),
            dqs => rc1_ram(15 downto 8),
            sa => sa1,
            scl => scl,
            sda => sda);

    trace_model_2 : ddrsdram_dimm_trace_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank2.ra_width - 3,
            max_phys_bank => 4,
            dq_width => bank2.rd_width,
            registered => registered_dimms,
            t_trace_k => 0.5 ns,
            t_trace_ctl => 0.5 ns,
            t_trace_dm => 0.5 ns,
            t_trace_dq => 0.5 ns,
            t_trace_dqs => 0.5 ns)
        port map(
            mk0 => rc2(16),
            mk0_l => rc2(19),
            mk1 => rc2(17),
            mk1_l => rc2(20),
            mk2 => rc2(18),
            mk2_l => rc2(21),
            mcke => rc2(23 downto 22),
            mwe_l => rc2(24),
            mcas_l => rc2(25),
            mras_l => rc2(26),
            mcs_l => rc2(30 downto 27),
            mba => ra2(18 downto 16),
            ma => ra2(15 downto 0),
            mdq => rd2,
            mdm => rc2(7 downto 0),
            mdqs => rc2(15 downto 8),
            dk0 => rc2_ram(16),
            dk0_l => rc2_ram(19),
            dk1 => rc2_ram(17),
            dk1_l => rc2_ram(20),
            dk2 => rc2_ram(18),
            dk2_l => rc2_ram(21),
            dcke => rc2_ram(23 downto 22),
            dwe_l => rc2_ram(24),
            dcas_l => rc2_ram(25),
            dras_l => rc2_ram(26),
            dcs_l => rc2_ram(30 downto 27),
            dba => ra2_ram(18 downto 16),
            da => ra2_ram(15 downto 0),
            ddq => rd2_ram,
            ddm => rc2_ram(7 downto 0),
            ddqs => rc2_ram(15 downto 8));

    mem_model_2 : ddrsdram_dimm_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank2.ra_width - 3,
            max_phys_bank => 4,
            num_phys_bank => 1,
            dq_width => bank2.rd_width,
            registered => registered_dimms,
            extra_dqs => false)
        port map(
            k0 => rc2_ram(16),
            k0_l => rc2_ram(19),
            k1 => rc2_ram(17),
            k1_l => rc2_ram(20),
            k2 => rc2_ram(18),
            k2_l => rc2_ram(21),
            cke => rc2_ram(23 downto 22),
            we_l => rc2_ram(24),
            cas_l => rc2_ram(25),
            ras_l => rc2_ram(26),
            cs_l => rc2_ram(30 downto 27),
            ba => ra2_ram(18 downto 16),
            a => ra2_ram(15 downto 0),
            dq => rd2_ram,
            dm => rc2_ram(7 downto 0),
            dqs => rc2_ram(15 downto 8),
            sa => sa2,
            scl => scl,
            sda => sda);

    trace_model_3 : ddrsdram_dimm_trace_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank3.ra_width - 3,
            max_phys_bank => 4,
            dq_width => bank3.rd_width,
            registered => registered_dimms,
            t_trace_k => 0.5 ns,
            t_trace_ctl => 0.5 ns,
            t_trace_dm => 0.5 ns,
            t_trace_dq => 0.5 ns,
            t_trace_dqs => 0.5 ns)
        port map(
            mk0 => rc3(16),
            mk0_l => rc3(19),
            mk1 => rc3(17),
            mk1_l => rc3(20),
            mk2 => rc3(18),
            mk2_l => rc3(21),
            mcke => rc3(23 downto 22),
            mwe_l => rc3(24),
            mcas_l => rc3(25),
            mras_l => rc3(26),
            mcs_l => rc3(30 downto 27),
            mba => ra3(18 downto 16),
            ma => ra3(15 downto 0),
            mdq => rd3,
            mdm => rc3(7 downto 0),
            mdqs => rc3(15 downto 8),
            dk0 => rc3_ram(16),
            dk0_l => rc3_ram(19),
            dk1 => rc3_ram(17),
            dk1_l => rc3_ram(20),
            dk2 => rc3_ram(18),
            dk2_l => rc3_ram(21),
            dcke => rc3_ram(23 downto 22),
            dwe_l => rc3_ram(24),
            dcas_l => rc3_ram(25),
            dras_l => rc3_ram(26),
            dcs_l => rc3_ram(30 downto 27),
            dba => ra3_ram(18 downto 16),
            da => ra3_ram(15 downto 0),
            ddq => rd3_ram,
            ddm => rc3_ram(7 downto 0),
            ddqs => rc3_ram(15 downto 8));

    mem_model_3 : ddrsdram_dimm_model
        generic map(
            max_bank_width => 3,
            max_addr_width => bank3.ra_width - 3,
            max_phys_bank => 4,
            num_phys_bank => 1,
            dq_width => bank3.rd_width,
            registered => registered_dimms,
            extra_dqs => false)
        port map(
            k0 => rc3_ram(16),
            k0_l => rc3_ram(19),
            k1 => rc3_ram(17),
            k1_l => rc3_ram(20),
            k2 => rc3_ram(18),
            k2_l => rc3_ram(21),
            cke => rc3_ram(23 downto 22),
            we_l => rc3_ram(24),
            cas_l => rc3_ram(25),
            ras_l => rc3_ram(26),
            cs_l => rc3_ram(30 downto 27),
            ba => ra3_ram(18 downto 16),
            a => ra3_ram(15 downto 0),
            dq => rd3_ram,
            dm => rc3_ram(7 downto 0),
            dqs => rc3_ram(15 downto 8),
            sa => sa3,
            scl => scl,
            sda => sda);

    trace_model_4 : ddr2sram_trace_model
        generic map(
            a_width => bank4.ra_width,
            dq_width => bank4.rd_width,
            t_trace_k => 2.0 ns,
            t_trace_c => 2.0 ns,
            t_trace_cq => 2.0 ns,
            t_trace_a => 2.0 ns,
            t_trace_ctl => 2.0 ns,
            t_trace_dq => 2.0 ns)
        port map(
            mk => rc4(6),
            mk_l => rc4(7),
            mc => rc4(8),
            mc_l => rc4(9),
            mld_l => rc4(4),
            mw_l => rc4(5),
            mbwe_l => rc4(3 downto 0),
            mdq => rd4,
            mcq => open,
            mcq_l => open,
            ma => ra4,
            dk => rc4_ram(6),
            dk_l => rc4_ram(7),
            dc => rc4_ram(8),
            dc_l => rc4_ram(9),
            dld_l => rc4_ram(4),
            dw_l => rc4_ram(5),
            dbwe_l => rc4_ram(3 downto 0),
            ddq => rd4_ram,
            dcq => cq_ram(4),
            dcq_l => cq_ram_l(4),
            da => ra4_ram);

    mem_model_4 : ddr2sram_model
        generic map(
            burst_order => 2,
            a_width => 22,
            dq_width => 32)
        port map(
            k => rc4_ram(6),
            k_l => rc4_ram(7),
            c => rc4_ram(8),
            c_l => rc4_ram(9),
            ld_l => rc4_ram(4),
            w_l => rc4_ram(5),
            bwe_l => rc4_ram(3 downto 0),
            dq => rd4_ram,
            cq => cq_ram(4),
            cq_l => cq_ram_l(4),
            a => ra4_ram);

    UUT : memory
        generic map(
            bank0 => bank0,
            bank1 => bank1,
            bank2 => bank2,
            bank3 => bank3,
            bank4 => bank4,
            bank5 => bank5,
            bank6 => bank6,
            bank7 => bank7,
            bank8 => bank8,
            bank9 => bank9,
            bank10 => bank10,
            bank11 => bank11,
            bank12 => bank12,
            bank13 => bank13,
            bank14 => bank14,
            bank15 => bank15,
            num_ramclk => num_ramclk)
        port map(
            lclk       => lclk,
            lreset_l   => lreset_l,
            lads_l     => lads_l,
            lblast_l   => lblast_l,
            lbe_l      => lbe_l,
            lwrite     => lwrite,
            lad        => lad,
            lbterm_l   => lbterm_l,
            lready_l   => lready_l,
            fholda     => fholda,
            mclk       => mclk,
            refclk_p   => refclk_p,
            refclk_n   => refclk_n,
            ra0        => ra0,
            rc0        => rc0,
            rd0        => rd0,
            ra1        => ra1,
            rc1        => rc1,
            rd1        => rd1,
            ra2        => ra2,
            rc2        => rc2,
            rd2        => rd2,
            ra3        => ra3,
            rc3        => rc3,
            rd3        => rd3,
            ra4        => ra4,
            rc4        => rc4,
            rd4        => rd4,
            ra5        => ra5,
            rc5        => rc5,
            rd5        => rd5,
            ra6        => ra6,
            rc6        => rc6,
            rd6        => rd6,
            ra7        => ra7,
            rc7        => rc7,
            rd7        => rd7,
            ra8        => ra8,
            rc8        => rc8,
            rd8        => rd8,
            ra9        => ra9,
            rc9        => rc9,
            rd9        => rd9,
            ra10       => ra10,
            rc10       => rc10,
            rd10       => rd10,
            ra11       => ra11,
            rc11       => rc11,
            rd11       => rd11,
            ra12       => ra12,
            rc12       => rc12,
            rd12       => rd12,
            ra13       => ra13,
            rc13       => rc13,
            rd13       => rd13,
            ra14       => ra14,
            rc14       => rc14,
            rd14       => rd14,
            ra15       => ra15,
            rc15       => rc15,
            rd15       => rd15,
            ramclko    => ramclk,
            scl        => scl,
            sda        => sda);
         
    agent_ds : locbus_agent_mux32
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            lad         => lad,
            lbe_l       => lbe_l,
            lblast_l    => lblast_l,
            lbterm_l    => lbterm_l,
            lready_l    => lready_l,
            lwrite      => lwrite,
            lhold       => lhold(0),
            lholda      => lholda(0),
            bus_in      => bus_in,
            bus_out     => bus_out);

    arb_0 : locbus_arb
        generic map(
            n_arb       => num_agent,
            priority    => priorities)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lhold       => lhold,
            lholda      => lholda);

    lbpcheck_0 : lbpcheck
        generic map(
            multiplexed => true,
            wide        => false)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            l64_l       => l64_l,
            la          => lad(31 downto 2),
            lad_lo      => lad(31 downto 0),
            lad_hi      => lad(31 downto 0),
            lbe_lo_l    => lbe_l(3 downto 0),
            lbe_hi_l    => lbe_l(3 downto 0),
            lwrite      => lwrite,
            lblast_l    => lblast_l,
            lready_l    => lready_l,
            lbterm_l    => lbterm_l);

end behav;
