--
-- memclk-xrcp.vhd - Generate memory interface clocks (Virtex/Virtex-E)
--                   Also performs deskew of clocks driven to ZBT SRAM.
--
-- SYNTHESIZABLE
--
-- Targets models:
--
--   o ADM-XRC-P
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_unsigned.all;

-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity memclk is
    generic(
        num_ramclk        : in    natural);
    port(
        rst               : in    std_logic;
        mclk_i            : in    std_logic;
        memclk0           : out   std_logic;
        locked            : out   std_logic_vector(num_ramclk - 1 downto 0);
        ramclki           : in    std_logic_vector(num_ramclk - 1 downto 0);
        ramclko           : out   std_logic_vector(num_ramclk - 1 downto 0);
        memrst            : out   std_logic);
end entity;

architecture struct of memclk is

    signal locked_memclk : std_logic;

    signal logic0, logic1 : std_logic;

    signal ramclk_ibufg : std_logic_vector(num_ramclk - 1 downto 0);
    signal locked_ramclk : std_logic_vector(num_ramclk - 1 downto 0);

    component CLKDLL
        port(
            CLKIN         : in  std_logic; 
            CLKFB         : in  std_logic; 
            RST           : in  std_logic; 
            CLK0          : out std_logic;
            CLK90         : out std_logic;
            CLK180        : out std_logic;
            CLK270        : out std_logic;
            CLK2X         : out std_logic;
            CLKDV         : out std_logic;
            LOCKED        : out std_logic);
    end component;

    component IBUFG
        port(
            I : in  std_logic;
            O : out std_logic);
    end component;
    
    component BUFG
        port(
            I : in  std_logic;
            O : out std_logic);
    end component;
    
    attribute DLL_FREQUENCY_MODE : string;
    attribute DUTY_CYCLE_CORRECTION : string;
    attribute STARTUP_WAIT : string;
    attribute CLK_FEEDBACK : string;

    attribute DLL_FREQUENCY_MODE of dll_ramclk0 : label is "LOW";
    attribute DUTY_CYCLE_CORRECTION of dll_ramclk0 : label is "TRUE";
    attribute STARTUP_WAIT of dll_ramclk0 : label is "FALSE";
    attribute CLK_FEEDBACK of dll_ramclk0 : label is "1X";

begin
    
    --
    -- Define constant values
    --
    logic0 <= '0';
    logic1 <= '1';

    memclk0 <= mclk_i;
    locked <= locked_ramclk;
    memrst <= not AND_reduce(locked_ramclk);

    --
    -- Generate deskewed SSRAM clocks
    --

    ibufg_ramclk0 : IBUFG
        port map(
            I => ramclki(0),
            O => ramclk_ibufg(0));

    dll_ramclk0 : CLKDLL
        port map(
            CLKIN => mclk_i, 
            CLKFB => ramclk_ibufg(0),
            RST => rst,
            CLK0 => ramclko(0),
            CLK90 => open,
            CLK180 => open,
            CLK270 => open,
            CLK2X => open,
            CLKDV => open,
            LOCKED => locked_ramclk(0));

end architecture;
