--
-- lclk_buf.vhd - generate local bus clock for distribution within FPGA
--                using a clock buffer (no DCM or DLL).
--
-- SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_unsigned.all;

-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity lclk_buf is
    port(
        rst           : in    std_logic;
        lclk          : in    std_logic;
        clk           : out   std_logic;
        locked        : out   std_logic);
end entity;

architecture struct of lclk_buf is

    signal lclk_ibufg : std_logic;
    signal lclk_bufg : std_logic;

    signal count : std_logic_vector(4 downto 0);

    component IBUFG
        port(
            I : in  std_logic;
            O : out std_logic);
    end component;
    
    component BUFG
        port(
            I : in  std_logic;
            O : out std_logic);
    end component;
    
begin
    
    gen_count : process(rst, lclk_bufg)
    begin
        if rst = '1' then
            count <= (others => '0');
        elsif lclk_bufg'event and lclk_bufg = '1' then
            if count(count'high) = '0' then
                count <= count + 1;
            end if;
        end if;
    end process;

    locked <= not count(count'high - 1);

    clk <= lclk_bufg;

    ibufg_lclk : IBUFG
        port map(
            I => lclk,
            O => lclk_ibufg);

    bufg_lclk : BUFG
        port map(
            I => lclk_ibufg,
            O => lclk_bufg);

end architecture;
