--
-- user_defs_pkg-xrc5t1.vhd - Model-specific definitions for 'user_app' implementation that
--                            tests onboard memory.
--
-- SYNTHESIZABLE
--
-- (C) Copyright Alpha Data 2008
--
-- Targets models:
--
--   o ADM-XRC-5T1
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_unsigned.all;

library work;
use work.memory64_common.all;

package user_defs is

    type natural_vector_t is array(0 to max_num_bank - 1) of natural;
    type boolean_vector_t is array(0 to max_num_bank - 1) of boolean;

    -- Widths in logical bits of memory port data
    constant bank_width : natural_vector_t := (0 => 128, 1 => 128, 2 => 64, others => 0);

    -- Widths in logical bits of memory port byte enables
    constant bank_be_width : natural_vector_t := (0 => 16, 1 => 16, 2 => 8, others => 0);

    -- Each element should be true if the corresponding memory bank is physically present
    -- Allows banks to be skipped when a device is fitted that doesn't have enough I/O
    -- pins to support all banks of memory
    constant bank_present : boolean_vector_t := (0 => true, 1 => true, 2 => true, others => false);

    -- Width in bits of physical memory port data busses
    constant rd_width : natural_vector_t := (0 => 32, 1 => 32, 2 => 16, others => 0);

end;
