--
-- reario-xrc2.vhd - FPGA design to output a walking 1 on I/O pins
--
-- (C) Copyright Alpha Data Parallel Systems Ltd. 2002-2004
--
-- Example FPGA design for:
--
--   o ADM-XRCII
--
-- This example outputs a walking bit on the 64 user I/O pins.
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;
use IEEE.std_logic_arith.all;

entity reario is
    port(
        lclk:            in          std_logic;
        lreset_l:        in          std_logic;
        reario:          inout       std_logic_vector(63 downto 0));
end reario;

architecture rtl of reario is
    
    signal rst:         std_logic;
    signal shiftreg:    std_logic_vector(63 downto 0);

begin

    rst <= not lreset_l;

    reario(63 downto 0) <= shiftreg(63 downto 0);

    shift_pattern: process(lclk, rst)
    begin
        if rst = '1' then
            shiftreg <= EXT("1", 64);
        elsif lclk'event and lclk = '1' then
            shiftreg(63 downto 0) <= shiftreg(62 downto 0) & shiftreg(63);
        end if;
    end process;

end rtl;

