--
-- reario-xrcp.vhd - FPGA design to output a walking 1 on I/O pins
--
-- (C) Copyright Alpha Data Parallel Systems Ltd. 1999-2004
--
-- Example FPGA design for:
--
--   o ADM-XRC-P
--
-- This example outputs a walking bit on the 53 user I/O pins.
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;
use IEEE.std_logic_arith.all;

entity reario is
    port(
        lclk:            in          std_logic;
        lreset_l:        in          std_logic;
        reario:          inout       std_logic_vector(52 downto 0));
end reario;

architecture rtl of reario is
    
    signal rst:         std_logic;
    signal shiftreg:    std_logic_vector(51 downto 0);
    signal userclkin:   std_logic;

begin

    rst <= not lreset_l;
    userclkin <= reario(51);

    reario(50 downto 0) <= shiftreg(50 downto 0);
    reario(51) <= 'Z';
    reario(52) <= shiftreg(51);

    shift_pattern: process(lclk, rst)
    begin
        if rst = '1' then
            shiftreg <= EXT("1", 52);
        elsif lclk'event and lclk = '1' then
            shiftreg(51 downto 0) <= shiftreg(50 downto 0) & shiftreg(51);
        end if;
    end process;

end rtl;

