--
-- test_simple64-xpl.vhd - Testbench for entity 'simple64'
--
-- (C) Copyright Alpha Data 2005-2007
--
-- Example testbench for:
--
--   o ADM-XPL
--   o ADM-XP
--   o ADM-XRC-5LX
--   o ADM-XRC-5T1
--
-- NOT SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library std;
use std.textio.all;

library plxsim;
use plxsim.plxsim.all;

entity test_simple64 is
end test_simple64;

architecture behav of test_simple64 is

    constant period:     time     := 30 ns;

    constant priorities : integer_vector_t(0 to 0) := (others => 0);

    signal lclk, mclk:         std_logic := '1';
    signal lreset_l:           std_logic := '0';
    signal lbe_l:              std_logic_vector(7 downto 0) := (others => 'Z');
    signal lad:                std_logic_vector(63 downto 0) := (others => 'Z');
    signal lwrite:             std_logic := '0';
    signal lads_l:             std_logic := '1';
    signal l64_l:              std_logic := '1';
    signal lblast_l:           std_logic := '1';
    signal lbterm_l:           std_logic := 'Z';
    signal lready_l:           std_logic := 'Z';
    signal lhold, lholda:      std_logic_vector(0 downto 0);
    signal fholda:             std_logic := '0';

    signal bus_in : locbus_in_t;
    signal bus_out : locbus_out_t := init_locbus_out;

    signal logic0, logic1 : std_logic;

    procedure report_message(
        str : in string)
    is
        variable msg : line;
    begin
        write(msg, str);
        writeline(output, msg);
    end;

    component simple64
        port(
            lclk           : in    std_logic;
            lreset_l       : in    std_logic; 
            lwrite         : in    std_logic;
            lads_l         : in    std_logic;
            lblast_l       : in    std_logic;
            lbterm_l       : inout std_logic;
            lad            : inout std_logic_vector(63 downto 0);
            lready_l       : inout std_logic; 
            lbe_l          : in    std_logic_vector(7 downto 0);
            fholda         : in    std_logic);
    end component;

begin

    logic0 <= '0';
    logic1 <= '1';

    lclk <= not lclk after period / 2;

    lreset_l <= transport '1' after 50 ns;

    lads_l <= 'H';
    lblast_l <= 'H';
    lready_l <= 'H';
    lbterm_l <= 'H';
    lwrite <= 'H';

    stimulate: process
        variable data : byte_vector_t(0 to 127);
        variable be : byte_enable_t(0 to 127);
        variable n : natural;
    begin
        plxsim_wait_cycles(4, bus_in);

        be := (others => '1');

        plxsim_request_bus(true, bus_in, bus_out);

        -- Write a pattern to the registers
        data(0 to 7) := conv_byte_vector(X"deadbeefcafeface");
        plxsim_write(3, true, X"00000000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);
	report_message("Wrote 0x" & conv_string_hex(data(0 to 3)) & " to address 0x0");
	report_message("Wrote 0x" & conv_string_hex(data(4 to 7)) & " to address 0x4");

        -- Read back the registers
        plxsim_read(3, true, X"00000000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);
	report_message("Read 0x" & conv_string_hex(data(0 to 3)) & " from address 0x0");
	report_message("Read 0x" & conv_string_hex(data(4 to 7)) & " from address 0x4");

        plxsim_request_bus(false, bus_in, bus_out);

        report_message("Simulation of SIMPLE64 complete");

        wait;
    end process;

    UUT : simple64
        port map(
            lclk           => lclk,
            lreset_l       => lreset_l,
            lwrite         => lwrite,
            lads_l         => lads_l,
            lblast_l       => lblast_l,
            lbterm_l       => lbterm_l,
            lad            => lad,
            lready_l       => lready_l,
            lbe_l          => lbe_l,
            fholda         => fholda);

    arb_0 : locbus_arb
        generic map(
            n_arb       => 1,
            priority    => priorities)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lhold       => lhold,
            lholda      => lholda);

    agent_0 : locbus_agent_mux64
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            lad         => lad,
            lbe_l       => lbe_l,
            l64_l       => l64_l,
            lblast_l    => lblast_l,
            lbterm_l    => lbterm_l,
            lready_l    => lready_l,
            lwrite      => lwrite,
            lhold       => lhold(0),
            lholda      => lholda(0),
            bus_in      => bus_in,
            bus_out     => bus_out);

    lbpcheck_0 : lbpcheck
        generic map(
            multiplexed => true,
            wide        => true)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            l64_l       => l64_l,
            la          => lad(31 downto 2),
            lad_lo      => lad(31 downto 0),
            lad_hi      => lad(63 downto 32),
            lbe_lo_l    => lbe_l(3 downto 0),
            lbe_hi_l    => lbe_l(7 downto 4),
            lwrite      => lwrite,
            lblast_l    => lblast_l,
            lready_l    => lready_l,
            lbterm_l    => lbterm_l);

end behav;
