--
-- test_zbt-xrcp.vhd - Testbench for entity 'zbt'
--
-- (C) Copyright Alpha Data 2004-2005
--
-- Example testbench for:
--
--   o ADM-XRC-P
--
-- NOT SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library std;
use std.textio.all;

library plxsim;
use plxsim.plxsim.all;

entity test_zbt is
end test_zbt;

architecture behav of test_zbt is

    --
    -- These parameters describe the ADM-XRC-P.
    --
    constant max_bank: natural  := 8;
    constant num_bank: natural  := 4;
    constant num_clock: natural  := 1;
    constant ctl_width: natural  := 9;
    constant addr_width: natural  := 19;
    constant data_width: natural  := 32;
    constant bank_group: natural  := 1;

    constant period:     time     := 30 ns;

    constant pctl_width:  natural := ctl_width  * bank_group;
    constant paddr_width: natural := addr_width * bank_group;
    constant pdata_width: natural := data_width * bank_group;

    constant read_latency: natural := 7;

    constant priorities : integer_vector_t(0 to 0) := (others => 0);

    signal lclk, mclk:         std_logic := '1';
    signal ramclk:             std_logic_vector(num_clock - 1 downto 0);
    signal lreset_l:           std_logic := '0';
    signal la:                 std_logic_vector(31 downto 2) := (others => 'Z');
    signal lbe_l:              std_logic_vector(3 downto 0) := (others => 'Z');
    signal ld:                 std_logic_vector(31 downto 0) := (others => 'Z');
    signal lwrite:             std_logic := '0';
    signal lads_l:             std_logic := '1';
    signal lblast_l:           std_logic := '1';
    signal lbterm_l:           std_logic := 'Z';
    signal lready_l:           std_logic := 'Z';
    signal lhold, lholda:      std_logic_vector(0 downto 0);
    signal ra0, ra1, ra2, ra3: std_logic_vector(paddr_width - 1 downto 0) := (others => 'Z');
    signal ra4, ra5, ra6, ra7: std_logic_vector(paddr_width - 1 downto 0) := (others => 'Z');
    signal rc0, rc1, rc2, rc3: std_logic_vector(pctl_width  - 1 downto 0) := (others => 'H');
    signal rc4, rc5, rc6, rc7: std_logic_vector(pctl_width  - 1 downto 0) := (others => 'H');
    signal rd0, rd1, rd2, rd3: std_logic_vector(pdata_width - 1 downto 0) := (others => 'Z');
    signal rd4, rd5, rd6, rd7: std_logic_vector(pdata_width - 1 downto 0) := (others => 'Z');
    signal fholda:             std_logic := '0';

    signal pipeline:           natural := 0;
    signal rd_delay:           time := 0 ns;

    type ra_type is array(0 to max_bank - 1) of std_logic_vector(paddr_width - 1 downto 0);
    type rc_type is array(0 to max_bank - 1) of std_logic_vector(pctl_width  - 1 downto 0);
    type rd_type is array(0 to max_bank - 1) of std_logic_vector(pdata_width - 1 downto 0);
    signal del_ra: ra_type;
    signal del_rc: rc_type;
    signal rc_vec : rc_type;
    signal rd_vec : rd_type;

    signal bus_in : locbus_in_t;
    signal bus_out : locbus_out_t := init_locbus_out;

    signal logic0, logic1 : std_logic;

    procedure report_message(
        str : in string)
    is
        variable msg : line;
    begin
        write(msg, str);
        writeline(output, msg);
    end;

    component zbt
        generic(
            num_bank       : in    natural := 4;
            num_clock      : in    natural := 1;
            addr_width     : in    natural := 19;
            ctl_width      : in    natural := 9;
            data_width     : in    natural := 32;
            bank_group     : in    natural := 1);
        port(
            lclk           : in    std_logic;
            mclk           : in    std_logic;
            ramclko        : out   std_logic_vector(num_clock - 1 downto 0);
            ramclki        : in    std_logic_vector(num_clock - 1 downto 0);
            lreset_l       : in    std_logic; 
            lwrite         : in    std_logic;
            lads_l         : in    std_logic;
            lblast_l       : in    std_logic;
            lbterm_l       : inout std_logic;
            ld             : inout std_logic_vector(31 downto 0);
            la             : in    std_logic_vector(23 downto 2);
            lready_l       : inout std_logic; 
            lbe_l          : in    std_logic_vector(3 downto 0);
            rc0            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra0            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd0            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc1            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra1            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd1            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc2            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra2            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd2            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc3            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra3            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd3            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc4            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra4            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd4            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc5            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra5            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd5            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc6            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra6            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd6            : inout std_logic_vector(pdata_width - 1 downto 0);
            rc7            : out   std_logic_vector(pctl_width  - 1 downto 0);
            ra7            : out   std_logic_vector(paddr_width - 1 downto 0);
            rd7            : inout std_logic_vector(pdata_width - 1 downto 0);
            fholda         : in    std_logic);
    end component;

begin

    logic0 <= '0';
    logic1 <= '1';

    lclk <= not lclk after period / 2;
    mclk <= not mclk after 17 ns;

    lreset_l <= transport '1' after 50 ns;

    lads_l <= 'H';
    lblast_l <= 'H';
    lready_l <= 'H';
    lbterm_l <= 'H';
    lwrite <= 'H';

    rd_delay <= pipeline * period + 1 ns;

    del_rc <= transport (rc0, rc1, rc2, rc3, rc4, rc5, rc6, rc7) after rd_delay + period;
    del_ra <= transport (ra0, ra1, ra2, ra3, ra4, ra5, ra6, ra7) after rd_delay + period;
    rc_vec <= (rc0, rc1, rc2, rc3, rc4, rc5, rc6, rc7);
    rd_vec <= (rd0, rd1, rd2, rd3, rd4, rd5, rd6, rd7);

    zbt_stim: process(lreset_l, lclk)
        variable arange_hi: natural;
        variable arange_lo: natural;
        variable crange_hi: natural;
        variable crange_lo: natural;
        variable drange_hi: natural;
        variable drange_lo: natural;
    begin
        if lclk'event and lclk = '1' then
            for i in 0 to bank_group - 1 loop
                arange_hi := addr_width * (i + 1) - 1;
                arange_lo := addr_width * i;
                crange_hi := ctl_width * i + data_width / 8 + 1;
                crange_lo := ctl_width * i + data_width / 8;
                drange_hi := data_width * (i + 1) - 1;
                drange_lo := data_width * i;

                if rc0(crange_hi downto crange_lo) = "01" then
                    rd0(drange_hi downto drange_lo) <= transport
                       EXT(ra0(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd0 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc1(crange_hi downto crange_lo) = "01" then
                    rd1(drange_hi downto drange_lo) <= transport
                        EXT(ra1(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd1 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc2(crange_hi downto crange_lo) = "01" then
                    rd2(drange_hi downto drange_lo) <= transport
                        EXT(ra2(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd2 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc3(crange_hi downto crange_lo) = "01" then
                    rd3(drange_hi downto drange_lo) <= transport
                        EXT(ra3(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd3 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc4(crange_hi downto crange_lo) = "01" then
                    rd4(drange_hi downto drange_lo) <= transport
                        EXT(ra4(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd4 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc5(crange_hi downto crange_lo) = "01" then
                    rd5(drange_hi downto drange_lo) <= transport
                        EXT(ra5(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd5 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc6(crange_hi downto crange_lo) = "01" then
                    rd6(drange_hi downto drange_lo) <= transport
                        EXT(ra6(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd6 <= transport (others => 'Z') after rd_delay;
                end if;

                if rc7(crange_hi downto crange_lo) = "01" then
                    rd7(drange_hi downto drange_lo) <= transport
                        EXT(ra7(arange_hi downto arange_lo), data_width) after rd_delay;
                else
                    rd7 <= transport (others => 'Z') after rd_delay;
                end if;
           end loop;
        end if;
    end process;

    stimulate: process
        variable data : byte_vector_t(0 to 127);
        variable be : byte_enable_t(0 to 127);
        variable n : natural;
    begin
        plxsim_wait_cycles(4, bus_in);

        be := (others => '1');

        plxsim_request_bus(true, bus_in, bus_out);

        --
        -- Go to flowthrough mode
        --
        pipeline <= 0;
        data(4 to 4) := conv_byte_vector(X"00");
        plxsim_write(2, true, X"00000004", be(4 to 4), data(4 to 4), n, bus_in, bus_out);

        --
        -- Set SSRAM size to 256k x n.
        -- Results in a bank size of 1MByte, 2 banks per page.
        --
        data(8 to 8) := conv_byte_vector(X"01");
        plxsim_write(2, true, X"8", be(8 to 8), data(8 to 8), n, bus_in, bus_out);

        --
        -- Read SSRAM bank 0 up to bank boundary; should not read past the bank
        -- boundary on the local bus due to LBTERM# being asserted by the FPGA.
        --
        plxsim_read(2, false, X"002fffec", be(0 to 63), data(0 to 63), n, bus_in, bus_out);

        --
        -- Read SSRAM bank 1 at beginning of bank.
        --
        plxsim_read(2, true, X"00300000", be(0 to 35), data(0 to 35), n, bus_in, bus_out);

        --
        -- Set page register to 0x1, allowing access to SSRAM banks 2 & 3.
        --
        data(0 to 0) := conv_byte_vector(X"01");
        plxsim_write(2, true, X"0", be(0 to 0), data(0 to 0), n, bus_in, bus_out);

        --
        -- Write SSRAM bank 3.
        --
        data(0 to 15) := conv_byte_vector(X"89abcdef01234567deadbeefcafeface");
        plxsim_write(2, true, X"003001d4", be(0 to 15), data(0 to 15), n, bus_in, bus_out);

        plxsim_request_bus(false, bus_in, bus_out);

        report_message("Simulation of ZBT complete");

        wait;
    end process;

    UUT : zbt
        port map(
            lclk           => lclk,
            mclk           => mclk,
            ramclko        => ramclk,
            ramclki        => ramclk,
            lreset_l       => lreset_l,
            lwrite         => lwrite,
            lads_l         => lads_l,
            lblast_l       => lblast_l,
            lbterm_l       => lbterm_l,
            ld             => ld,
            la             => la(23 downto 2),
            lready_l       => lready_l,
            lbe_l          => lbe_l,
            rc0            => rc0,
            ra0            => ra0,
            rd0            => rd0,
            rc1            => rc1,
            ra1            => ra1,
            rd1            => rd1,
            rc2            => rc2,
            ra2            => ra2,
            rd2            => rd2,
            rc3            => rc3,
            ra3            => ra3,
            rd3            => rd3,
            rc4            => rc4,
            ra4            => ra4,
            rd4            => rd4,
            rc5            => rc5,
            ra5            => ra5,
            rd5            => rd5,
            rc6            => rc6,
            ra6            => ra6,
            rd6            => rd6,
            rc7            => rc7,
            ra7            => ra7,
            rd7            => rd7,
            fholda         => fholda);

    arb_0 : locbus_arb
        generic map(
            n_arb       => 1,
            priority    => priorities)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lhold       => lhold,
            lholda      => lholda);

    agent_0 : locbus_agent_nonmux
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            la          => la,
            lbe_l       => lbe_l,
            lblast_l    => lblast_l,
            ld          => ld,
            lbterm_l    => lbterm_l,
            lready_l    => lready_l,
            lwrite      => lwrite,
            lhold       => lhold(0),
            lholda      => lholda(0),
            bus_in      => bus_in,
            bus_out     => bus_out);

    lbpcheck_0 : lbpcheck
        generic map(
            multiplexed => false,
            wide        => false)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            l64_l       => lads_l,
            la          => la,
            lad_lo      => ld,
            lad_hi      => ld,
            lbe_lo_l    => lbe_l,
            lbe_hi_l    => lbe_l,
            lwrite      => lwrite,
            lblast_l    => lblast_l,
            lready_l    => lready_l,
            lbterm_l    => lbterm_l);

end behav;
