--
-- zbt-xrc4sx.vhd - Wrapper for zbt_main module, targetted for ADM-XRC4SX card.
--
-- (C) Copyright Alpha Data Parallel Systems Ltd. 2005
--
-- Example program for:
--
--   o ADM-XRC4SX
--
-- This module wraps up the 'zbt_main' module to make it compatible with
-- the pinout of the ADM-XRC4SX.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity zbt is
    generic(
        num_bank         : in    natural := 4;
        num_clock        : in    natural := 4;
        addr_width       : in    natural := 21;
        ctl_width        : in    natural := 8;
        data_width       : in    natural := 32;
        bank_group       : in    natural := 1);
    port(
        lclk             : in    std_logic;
        mclk_p           : in    std_logic;
        mclk_n           : in    std_logic;
        ramclko          : out   std_logic_vector(num_clock - 1 downto 0);
        ramclki          : in    std_logic_vector(num_clock - 1 downto 0);
        lreset_l         : in    std_logic; 
        lwrite           : in    std_logic;
        lads_l           : in    std_logic;
        lblast_l         : in    std_logic;
        lbterm_l         : inout std_logic;
        ld               : inout std_logic_vector(31 downto 0);
        la               : in    std_logic_vector(23 downto 2);
        lready_l         : inout std_logic; 
        lbe_l            : in    std_logic_vector(3 downto 0);
        fholda           : in    std_logic;
        rc0              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra0              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd0              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc1              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra1              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd1              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc2              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra2              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd2              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc3              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra3              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd3              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc4              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra4              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd4              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc5              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra5              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd5              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc6              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra6              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd6              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
        rc7              : out   std_logic_vector(ctl_width  * bank_group - 1 downto 0);
        ra7              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
        rd7              : inout std_logic_vector(data_width * bank_group - 1 downto 0));
end zbt;

architecture struct of zbt is

    signal ld_o : std_logic_vector(31 downto 0);
    signal ld_oe_l : std_logic_vector(31 downto 0);
    signal mclk : std_logic;

    component zbt_main is
        generic(
            la_top           : in    natural;
            num_bank         : in    natural;
            num_clock        : in    natural;
            addr_width       : in    natural;
            ctl_width        : in    natural;
            data_width       : in    natural;
            bank_group       : in    natural);
        port(
            lclk             : in    std_logic;
            mclk             : in    std_logic;
            ramclko          : out   std_logic_vector(num_clock - 1 downto 0);
            ramclki          : in    std_logic_vector(num_clock - 1 downto 0);
            lreset_l         : in    std_logic; 
            lwrite           : in    std_logic;
            lads_l           : in    std_logic;
            lblast_l         : in    std_logic;
            lbterm_l         : inout std_logic;
            la_i             : in    std_logic_vector(la_top downto 2);
            ld_i             : in    std_logic_vector(31 downto 0);
            ld_o             : out   std_logic_vector(31 downto 0);
            ld_oe_l          : out   std_logic_vector(31 downto 0);
            lready_l         : inout std_logic; 
            lbe_l            : in    std_logic_vector(3 downto 0);
            fholda           : in    std_logic;
            rc0              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra0              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd0              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc1              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra1              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd1              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc2              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra2              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd2              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc3              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra3              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd3              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc4              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra4              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd4              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc5              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra5              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd5              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc6              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra6              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd6              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            rc7              : out   std_logic_vector(ctl_width * bank_group - 1 downto 0);
            ra7              : out   std_logic_vector(addr_width * bank_group - 1 downto 0);
            rd7              : inout std_logic_vector(data_width * bank_group - 1 downto 0);
            glob_lbo_l       : out   std_logic;
            glob_flothru_l   : out   std_logic);
    end component;

    component IBUFGDS
        port(
            I  : in  std_logic;
            IB : in  std_logic;
            O  : out std_logic);
    end component;

begin

    gen_ld : for i in ld'range generate
        ld(i) <= ld_o(i) when ld_oe_l(i) = '0' else 'Z';
    end generate;

    ibufgds_mclk : IBUFGDS
        port map(
            I => mclk_p,
            IB => mclk_n,
            O => mclk);

    zbt_main_0: zbt_main
        generic map(
            la_top         => 23,
            num_bank       => num_bank,
            num_clock      => num_clock,
            addr_width     => addr_width,
            ctl_width      => ctl_width,
            data_width     => data_width,
            bank_group     => bank_group)
        port map(
            lclk           => lclk,
            mclk           => mclk,
            ramclko        => ramclko,
            ramclki        => ramclki,
            lreset_l       => lreset_l,
            lwrite         => lwrite,
            lads_l         => lads_l,
            lblast_l       => lblast_l,
            lbterm_l       => lbterm_l,
            la_i           => la(23 downto 2),
            ld_i           => ld,
            ld_o           => ld_o,
            ld_oe_l        => ld_oe_l,
            lready_l       => lready_l,
            lbe_l          => lbe_l,
            fholda         => fholda,
            ra0            => ra0,
            rc0            => rc0,
            rd0            => rd0,
            ra1            => ra1,
            rc1            => rc1,
            rd1            => rd1,
            ra2            => ra2,
            rc2            => rc2,
            rd2            => rd2,
            ra3            => ra3,
            rc3            => rc3,
            rd3            => rd3,
            ra4            => ra4,
            rc4            => rc4,
            rd4            => rd4,
            ra5            => ra5,
            rc5            => rc5,
            rd5            => rd5,
            ra6            => ra6,
            rc6            => rc6,
            rd6            => rd6,
            ra7            => ra7,
            rc7            => rc7,
            rd7            => rd7);

end struct;
